from urllib.parse import urljoin

class Environment(object):

    _env = None

    def __init__(self, config):
        Environment._env = self
        self.config = GlobalConfig()
        if config is not None:
            self.config.__dict__.update(config)       

        self.market_api = None

        self.basicdata_api = None

        self.trade_api = None

        self.acct_info = None

        self.scheduler = None

    def get_instance():
        """
        返回已经创建的 Environment 对象
        """
        if Environment._env is None:
            raise RuntimeError(u"Environment has not been created.")
        return Environment._env

    def get_apiurl(self,relativeUrl):
        if self.config.runtime == 'DEBUG':
            return urljoin("https://dev_apigateway.inquantstudio.com/", relativeUrl)
        return urljoin("https://apigateway.inquantstudio.com/", relativeUrl)

    def get_quoteaddr(self):
        if self.config.runtime == 'DEBUG':
            return "wss://dev_quotegateway.inquantstudio.com"
        return "wss://quotegateway.inquantstudio.com"

    def get_trademsgaddr(self):
        if self.config.runtime == 'DEBUG':
            return "wss://dev_apigateway.inquantstudio.com/msg/"
        return "wss://apigateway.inquantstudio.com/msg/"

    def get_livetradeaddr(self):
        if self.config.runtime == 'DEBUG':
            return "https://dev_trade.inquant.cn/tradeAcc/"
        return "https://trade.inquant.cn/tradeAcc/"

class GlobalConfig(object):
    """全局配置"""

    def __init__(self):

        """运行时 DEBUG RELEASE"""
        self.runtime = "RELEASE"

        """日志文件
        {
            level = INFO,
            file = '/home/admin/logs/1.log'
        }
        """
        self.log = None

        """交易类型 BACKTEST PAPER_TRADING LIVE_TRADING"""
        self.run_type = None

        """外部通知配置
            {
                "type":"websocket",
                "address":"ws://localhost:9090/"
            }
        """
        self.ext_notify = None

        """"账户
            模拟盘:
            {
                "strategy_id":"1021",
                "account":"2012"
            }
            实盘:
            {
                "strategy_id":"1021",
                "account":"10010101",
                "password":"pwd",
                "comp_counter":100,
                "broker_type":12
            }
        """
        self.acct_info = None


# -*- coding: utf-8 -*-
from iqsopenapi.trade.papertrade.message.IMessage import *
from iqsopenapi.trade.papertrade.message.HeartBeatReq import *
from iqsopenapi.trade.papertrade.message.ReplyResp import *



# -*- coding: utf-8 -*-

from iqsopenapi.core.events import *
from iqsopenapi.environment import Environment
from iqsopenapi.core.Scheduler import *
import datetime

class Strategy(object):
    def __init__(self, event_bus,scope, ucontext):
        self.__user_context = ucontext
        self.__event_bus = event_bus
        self.__init = scope.get('on_init', None)
        self.__handle_bar = scope.get('on_bar', None)
        self.__handle_tick = scope.get('on_tick', None)
        self.__handle_order = scope.get('on_order', None)
        self.__handle_trade = scope.get('on_trade', None)
        self.__handle_open = scope.get('on_open', None)
        self.__handle_close = scope.get('on_close', None)

        if self.__init is not None:
            event_bus.add_listener(EVENT.On_Init, self.init)
        if self.__handle_tick is not None:
            event_bus.add_listener(EVENT.On_Tick, self.handle_tick)
        if self.__handle_bar is not None:
            event_bus.add_listener(EVENT.On_Bar, self.handle_bar)
        if self.__handle_order is not None:
            event_bus.add_listener(EVENT.On_Order, self.handle_order)
        if self.__handle_trade is not None:
            event_bus.add_listener(EVENT.On_Trade, self.handle_trade)
        if self.__handle_open is not None:
            Environment.get_instance().scheduler.add_task(SchedulerTask(None,datetime.time(20,30),EVENT.On_Open))
            event_bus.add_listener(EVENT.On_Open, self.handle_open)
        if self.__handle_close is not None:
            Environment.get_instance().scheduler.add_task(SchedulerTask(None,datetime.time(15,30),EVENT.On_Close))
            event_bus.add_listener(EVENT.On_Close, self.handle_close)
        event_bus.add_listener(EVENT.On_Scheduler, self.handle_scheduler)
        
    def user_context(self):
        return self.__user_context

    def init(self):
        logger.info("begin init")
        self.__init(self.__user_context)
        logger.info("init complated")

    def handle_bar(self, event):
        self.__rebuild_event_data(event.data)
        self.__handle_bar(self.__user_context, event.data)

    def handle_tick(self, event):
        self.__rebuild_event_data(event.data)
        self.__handle_tick(self.__user_context, event.data)

    def handle_order(self, event):
        self.__rebuild_event_data(event.data)
        self.__handle_order(self.__user_context, event.data)

    def handle_trade(self, event):
        self.__rebuild_event_data(event.data)
        self.__handle_trade(self.__user_context, event.data)

    def __rebuild_event_data(self, event_data):
        if event_data is None:
            return
        if hasattr(event_data,'symbol') and '.' not in event_data.symbol:
                event_data.symbol = '{0}.{1}'.format(event_data.symbol, event_data.exchange.name)

    def handle_open(self, event):
        if event is None or event.raise_time is None:
            return
        begin = event.raise_time
        end = begin + datetime.timedelta(days=1)
        open_times = Environment.get_instance().basicdata_api.GetOpenTimes(begin,end)
        if open_times is None or len(open_times) <= 0:
            return
        raise_time = event.raise_time + datetime.timedelta(hours=13)
        for x in open_times:
            if raise_time >= datetime.datetime.strptime(x.begin,'%Y%m%d%H%M%S') and raise_time <= datetime.datetime.strptime(x.end,'%Y%m%d%H%M%S'):
                logger.info("begin on_open")
                self.__handle_open(self.__user_context)
                logger.info("on_open complated")
                return

    def handle_close(self, event):
        if event is None or event.raise_time is None:
            return
        begin = event.raise_time + datetime.timedelta(days=-1)
        end = event.raise_time
        open_times = Environment.get_instance().basicdata_api.GetOpenTimes(begin,end)
        if open_times is None or len(open_times) <= 0:
            return
        raise_time = event.raise_time + datetime.timedelta(hours=-1)
        for x in open_times:
            if raise_time >= datetime.datetime.strptime(x.begin,'%Y%m%d%H%M%S') and raise_time <= datetime.datetime.strptime(x.end,'%Y%m%d%H%M%S'):
                logger.info("begin on_close")
                self.__handle_close(self.__user_context)
                logger.info("on_close complated")
                return

    def handle_scheduler(self, event):
        if event is None or event.func is None:
            return
        event.func(*event.args)

if __name__ == '__main__':

    from iqsopenapi import *
    from iqsopenapi.core import *

    ucontext = StrategyContext()

    event_bus = EventBus()

    loader = FileStrategyLoader(r'D:\gitwork\inquantstudio\IQSOpenApi\IQS.OpenApi.Python\iqsopenapi\example\teststrategy.py')

    from copy import copy

    scope = copy(iqsopenapi.__dict__)

    loader.load(scope)

    user_strategy = Strategy(event_bus, scope, ucontext)

    user_strategy.init()    

    marketApi = MarketApi(event_bus)

    marketApi.Init()

    event_bus.start()

    ret1 = marketApi.Subscribe("rb2010.SHFE.TICK.0","rb2010.SHFE.BAR.300","rb2005.SHFE.TICK.0")

    pass


# -*- coding: utf-8 -*- 
from enum import IntEnum

class IMessage(object):
    """消息"""

    def __init__(self):
        pass

class IRequestMessage(IMessage):
    """请求消息"""

    def __init__(self):
        """消息类型"""
        self.RequestType = RequestType.HeartBeat

        """请求编号"""
        self.ReqID  = ""

class RequestMessage(IRequestMessage):
    """请求消息"""

    def __init__(self):
        """消息类型"""
        self.RequestType = RequestType.HeartBeat

        """请求编号"""
        self.ReqID = ""

class IResponseMessage(IMessage):
    """应答消息"""

    def __init__(self):
        """消息类型"""
        self.ResponseType = ResponseType.HeartBeat

class ResponseMessage(IResponseMessage):
    """应答消息"""
    
    def __init__(self):
        """消息类型"""
        self.ResponseType = ResponseType.HeartBeat

        """数据"""
        self.Data = None
    

class RequestType(IntEnum):
    """行情消息类型"""
    
    """心跳包消息"""
    HeartBeat = 0

    """订阅"""
    Subscribe = 1

class ResponseType(IntEnum):
    """响应消息类型"""

    """心跳包消息"""
    HeartBeat = 0 

    """请求回复消息"""
    Reply = 1

    """委托回报"""
    OrderChange  = 2

    """成交回报"""
    ExecutionReport = 3



# -*- coding: utf-8 -*-
class ITradeApi(object):
    """交易接口"""

    def __init__(self,event_bus):
        """构造函数"""
        pass

    def Init(self):
        """初始化"""
        pass

    def SendOrder(self,symbol, exchange, orderSide, price, quantity, orderType, offset):
        """下单"""
        pass

    def CancelOrder(self,order):
        """撤单"""
        pass

    def GetAccount(self):
        """获取账户信息"""
        pass

    def GetOrder(self,orderId):
        """获取指定id的委托"""
        pass

    def GetOpenOrders(self):
        """获取打开的订单"""
        pass

    def GetOrders(self):
        """获取当日委托"""
        pass

    def GetPositions(self):
        """获取持仓"""
        pass

    def GetTrades(self):
        """获取当日成交"""
        pass


# -*- coding: utf-8 -*-
from iqsopenapi.core.events import *
from iqsopenapi.environment import Environment
from iqsopenapi.util import *
import datetime
import json
import time
import uuid

class ExternalNotify(object):
    """外部系统消息通知"""

    def __init__(self, addr, event_bus):
        """构造函数"""

        cfg = websocketCfg()
        cfg.address = addr
        cfg.hbContent = json.dumps({ 'requestType' : 'HeartBeat','reqID' : '' })
        cfg.recvCallback = self.__OnRecv
        cfg.connectedCallback = self.__OnConnect
        self.__wsClient = WebSocketClient(cfg)
        
        event_bus.add_listener(EVENT.On_Trade, self.handle_trade)
        event_bus.add_listener(EVENT.On_Order, self.handle_order)

    def init(self):
        """初始化"""
        if not self.__wsClient.Connect():
            logger.error("user notification connect failed!")
            return False
        self.__wsClient.AutoHeartBeat()
        return True

    def handle_order(self,event):
        if not event or not event.data:
            return
        dict = event.data.__dict__
        dict['requestType'] = 'OnOrder'
        message = json.dumps(dict,ensure_ascii=False, cls=DateTimeEncoder)
        if not self.__Send(message):
            logger.error("send order notify info fail :{0}".format(message))


    def handle_trade(self, event):
        """成交信号"""
        if not event or not event.data:
            return
        dict = event.data.__dict__
        dict['requestType'] = 'OnTrade'
        message = json.dumps(dict,ensure_ascii=False, cls=DateTimeEncoder)
        if not self.__Send(message):
            logger.error("send trade notify info fail :{0}".format(message))

    def __Send(self,msg):
        """消息发送"""
        if not msg:
            return False
        if not self.__wsClient.Send(msg):
            logger.error("send fail:{0}".format(msg))
            return False
        return True

    def __OnRecv(self,msg):
       '''收到消息'''
       if not msg:
           return
       responseType = msg.get("responseType")
       if responseType == 'HeartBeat':
           logger.info("收到心跳消息")
           return 

    def __OnConnect(self):
        """连接"""
        logger.info("连接成功，开始推送用户消息");


 




class ResultInfo(object):
    """返回结果"""

    def __init__(self,error_no=0,error_info="",data=None):

        """错误码"""
        self.error_no = error_no

        """错误信息"""
        self.error_info = error_info

        """数据"""
        self.data = data




# -*- coding: utf-8 -*-

from iqsopenapi.models.Account import *
from iqsopenapi.models.BarData import *
from iqsopenapi.models.Contract import *
from iqsopenapi.models.Order import *
from iqsopenapi.models.Position import *
from iqsopenapi.models.TickData import *
from iqsopenapi.models.ResultInfo import *
from iqsopenapi.models.Trade import *



# -*- coding: utf-8 -*-
from Crypto.Cipher import DES3
from Crypto.Util.Padding import *

__key_bytes = bytes("niugu123niugu456niugu123",encoding="utf-8")
__iv_bytes = bytes("12312300", encoding='utf-8')

def encrypt(plainText):
    """根据明文，做DES加密，并返回Hex字符串"""
    data = bytes(plainText,'utf-8')
    text = pad(data,8)
    cipher = DES3.new(__key_bytes, DES3.MODE_CBC,IV=__iv_bytes)
    m = cipher.encrypt(text)
    s = bytes.hex(m)
    return s

def decrypt(encryptText):
    data = bytes.fromhex(encryptText)
    cipher = DES3.new(__key_bytes, DES3.MODE_CBC,IV=__iv_bytes)
    s = cipher.decrypt(data)
    s = unpad(s,8)
    s = s.decode('utf-8') # unpad and decode bytes to str
    return s

if __name__ == '__main__':
    encryptTxt = "867a6d7640da567387b1343a4ca4067aa24204aed8091440877378ace6656a99313dde056406d33f"
    print(encryptTxt)
    val = decrypt(encryptTxt)    
    print(val)
    en = encrypt(val)
    print(en);


# -*- coding: utf-8 -*-

from iqsopenapi import *


def init(context):
    logger.info("init")
    context.s1 = "rb2010.SHFE"

def handle_tick(context,tick):
    logger.info('from func tick:{0},{1},{2}'.format(tick.Symbol,tick.LocalTime,tick.LastPx))

def handle_bar(context, bar):
    logger.info('from func bar:{0},{1},{2}'.format(bar.Symbol,bar.LocalTime,bar.LastPx))


config = {
  "base": {
    "start_date": "2016-06-01",
    "end_date": "2016-12-01",
    "benchmark": "000300.XSHG",
    "accounts": {
      "stock": 100000
    }
  },
  "extra": {
    "log_level": "verbose",
  },
  "mod": {
    "sys_analyser": {
      "enabled": True,
      "plot": True
    }
  }
}

# 您可以指定您要传递的参数
run_func(init=init, handle_tick=handle_tick, handle_bar=handle_bar, config=config)



# -*- coding: utf-8 -*-
class IMarketApi(object):
    """行情接口"""

    def __init__(self,event_bus):
        """构造函数"""
        pass

    def Init(self,strategyID):
        """初始化"""
        pass

    def GetSubscibes(self):
        """获取订阅列表"""
        pass

    def Subscribe(self,*subInfos):
        """行情订阅"""
        pass

    def Unsubscribe(self,*subInfos):
        """取消订阅"""
        pass

    def GetHisBar(self, symbol, exchange, barType, startTime, endTime):
        """获取历史K线数据"""
        pass

    def GetLastBar(self, symbol, exchange, barType, count):
        """获取历史K线数据"""
        pass

    def GetHisTick(self, symbol, exchange, startTime, endTime):
        """获取历史TICK数据"""
        pass

    def GetLastTick(self, symbol, exchange, count):
        """获取历史TICK数据"""
        pass



# -*- coding: utf-8 -*- 

from datetime import *
from iqsopenapi.models.Contract import *

class BarData(object):
    """历史数据模型"""

    def __init__(self):

        """合约或股票代码名称"""
        self.symbol = ""

        """交易所"""
        self.exchange = Exchange.UnKnow

        """周期类型以秒计算"""
        self.bar_type = 0

        """时间"""
        self.local_time = datetime.now()

        """昨收"""
        self.pre_close = 0.0

        """开盘价"""
        self.open = 0.0

        """最高价"""
        self.high = 0.0

        """最低价"""
        self.low = 0.0

        """收盘价"""
        self.close = 0.0

        """成交量"""
        self.volume = 0

        """成交额"""
        self.turnover = 0.0      

        """持仓量"""
        self.open_interest = 0

        """结算价"""
        self.settlement = 0.0  


        

        

        

        

        

        





# -*- coding: utf-8 -*-
from enum import IntEnum

class TradeCounterType(IntEnum):
    ''' 柜台类型 '''

    ''' 默认-无 '''
    NONE=0
    ''' CTP '''
    CTP = 1
    ''' 恒生 '''
    HS = 2
    ''' 顶点 '''
    DD = 3
    ''' 金仕达 '''
    JSD = 4
    ''' 飞马 '''
    FM = 5
    
class ConOrderTimeValidTypeEnum(IntEnum):
    ''' 条件单查询日期类型 '''

    ''' 当前交易日有效 '''
    CurrentTradingDayValid=-1
    ''' 永久有效 '''
    ForverValid=0


# -*- coding: utf-8 -*-
import numpy as np
from iqsopenapi import *
import datetime
import talib

def on_init(ctx):
    #variables
    ctx.f1 = 5
    ctx.f2 = 0.2
    ctx.f3 = 0.2
    ctx.bet_size = 0.1
    ctx.symbol = 'rb'
    ctx.domi_contract = None
    ctx.last_domi = None

    positions = ctx.get_positions()

    position = ctx.get_position('rb2010.SHFE')

    # order = ctx.insert_order('m2009.DCE',OrderSide.Sell,OrderType.MKT,Offset.Open,0,8)
    # order = ctx.insert_order('m2009.DCE',OrderSide.Buy,OrderType.MKT,Offset.Open,0,5)

    order = ctx.insert_smart_order('p2009.DCE', -3)

    id = ctx.get_strategy_id()

    ctx.subscribe(('rb2010.SHFE','1m'),('jd2009.CFFEX','3m'))

    ctx.domi_contract = ctx.get_dominant_contract(ctx.symbol)
    conInfo = ctx.get_contract(ctx.domi_contract.symbol)

    last_bars = ctx.get_last_bars(ctx.domi_contract.symbol,'1d',300)

    last_ticks = ctx.get_last_ticks(ctx.domi_contract.symbol,100)

    account = ctx.get_account()

    orders = ctx.get_open_orders()

    positions = ctx.get_positions()

    trades = ctx.get_trades()

    order = ctx.insert_order('rb2010.SHFE',OrderSide.Buy,OrderType.MKT,Offset.Open,0,3)

    order = ctx.insert_smart_order('rb2010.SHFE', 1, 3508)

    #order1 = ctx.cancel_order(order)

    order2 = ctx.get_order(order.order_id)

    bars1 = ctx.get_history_bars('rb2010.SHFE','1m','20200521','20200522')

    bars2 = ctx.get_history_bars('rb2010.SHFE','1m',datetime(2020,5,21,9,10,0),datetime(2020,5,21,11,10,0))

    bars2 = ctx.get_history_bars('rb2010.SHFE','1m',date(2020,5,21),date(2020,5,22))
    
    ctx.run_by_time(run_time,datetime(2020,5,21,13,58,0),'CCC','DDDDD')

    ctx.run_by_time(run_time,time(13,50,0))

    ctx.insert_func_at_time(run_time,time(13,49,0))

    ctx.insert_func_at_time(run_time,datetime(2020,5,21,14,10,0),'AAA','BBBB')

    pass

def run_time(*args):
    pass
    
def on_open(ctx):
    ctx.domi_contract = ctx.get_dominant_contract(ctx.symbol)
    conInfo = ctx.get_contract(ctx.domi_contract.symbol)
    ctx.lots = conInfo.lots
    ctx.step = conInfo.step
    ctx.subscribe((ctx.domi_contract,'1m'))
    ctx.range = cal_range(ctx)
    ctx.prepared = False
    
def on_close(ctx):
    ctx.last_domi = ctx.domi_contract
    
def cal_range(ctx):
    last_bars = ctx.get_last_bars(ctx.domi_contract,'1d',300)
    High = np.array([bar.high for bar in last_bars])
    Low = np.array([bar.low for bar in last_bars])
    Close = np.array([bar.close for bar in last_bars])
    Range = talib.ATR(High,Low,Close,timeperiod=ctx.f1 - 1)[-1]
    return Range
    
def change_contract(ctx):
    lp = ctx.get_position(ctx.last_domi)
    quantity = (lp.quantity - lp.frozen) * lp.OrderSide
    if quantity:
        ctx.insert_smart_order(ctx.last_domi,-quantity)
        ctx.insert_smart_order(ctx.domi_contract,quantity)
        ctx.unsubscribe((ctx.last_domi,'1m'))

def on_tick(ctx,tick):
    logger.info('tick:{0},{1},{2}'.format(tick.symbol,tick.local_time,tick.last))

def on_order(ctx,order):
    logger.info('on order:{0},{1},{2},{3}'.format(order.symbol,order.side.name,order.offset.name,order.status.name))

def on_trade(ctx,trade):
    logger.info('on trade:{0},{1},{2},{3},{4}'.format(trade.symbol,trade.side.name,trade.offset.name,trade.quantity,trade.price))
    
def on_bar(ctx,bar):
    logger.info('bar:{0},{1},{2}'.format(bar.symbol,bar.local_time,bar.close))
    #if bar.symbol != ctx.domi_contract: return
    #if not ctx.prepared:
    #    if ctx.domi_contract != ctx.last_domi and ctx.last_domi:
    #        change_contract(ctx)
    #    ctx.open_price = bar.open
    #    ctx.up = ctx.open_price + ctx.f2 * ctx.range
    #    ctx.down = ctx.open_price - ctx.f3 * ctx.range
    #    ctx.up = round(ctx.up / ctx.step) * ctx.step
    #    ctx.down = round(ctx.down / ctx.step) * ctx.step
    #    ctx.last_high = bar.high
    #    ctx.last_low = bar.low
    #    ctx.prepared = True
    #    return
    #else:
    #    if not (bar.high > ctx.up and bar.low < ctx.down):
    #        if ctx.last_high <= ctx.up and bar.open >= ctx.up:
    #            acc = ctx.get_account()
    #            quantity = int(acc.available / ctx.up / ctx.lots / 0.01 *
    #            ctx.bet_size)
    #            if quantity:
    #                ctx.insert_smart_order(ctx.domi_contract,quantity)
    #        if ctx.last_low >= ctx.down and bar.open <= ctx.down:
    #            acc = ctx.get_account()
    #            quantity = int(acc.available / ctx.up / ctx.lots / 0.01 *
    #            ctx.bet_size)
    #            if quantity:
    #                ctx.insert_smart_order(ctx.domi_contract,-quantity)
    #    ctx.last_high = bar.high
    #    ctx.last_low = bar.low
config = {
    "runtime":"DEBUG",
    "run_type":"PAPER_TRADING",
    "log":{
            'level':'INFO',
            'file' :r"D:\logs\test\strategy.log",
        },
    #"ext_notify":{
    #        "type":"websocket",
    #        "address":"ws://localhost:14002/"
    #    },
    "acct_info":{
           "strategy_id":"21",
           "account":"10030"
       }
    # "acct_info":{
    #         "strategy_id":"1021",
    #         "account":"093959",
    #         "password":"640423",
    #         "comp_counter": 96,
    #         "broker_type":152
    #     }
}
run_func(**globals())


# -*- coding: utf-8 -*-

from queue import Queue
from threading import *
from iqsopenapi.util import *
from enum import Enum

class EventBus(object):
    def __init__(self):
        """初始化事件管理器"""
        self.__eventQueue = Queue()
        self.__active = False
        self.__thread = Thread(target = self.__run)
        self.__handlers = {}

    def __run(self):
        """引擎运行"""
        while self.__active == True:
            try:
                event = self.__eventQueue.get()  
                self.__event_process(event)
            except Exception as e:
                logger.exception(e)

    def __event_process(self, event):
        """处理事件"""
        if event.event_type in self.__handlers:
            for handler in self.__handlers[event.event_type]:
                handler(event)

    def start(self):
        """启动"""
        self.__active = True
        self.__thread.start()

    def stop(self):
        """停止"""
        self.__active = False
        self.__thread.join()

    def add_listener(self, event_type, handler):
        """绑定事件和监听器处理函数"""
        try:
            handlerList = self.__handlers[event_type]
        except KeyError:
            handlerList = []
            self.__handlers[event_type] = handlerList
        if handler not in handlerList:
            handlerList.append(handler)

    def remove_listener(self, event_type, handler):
        """移除监听器的处理函数"""
        try:
            handlerList = self.__handlers[event_type]
            if handler in handlerList:
                handlerList.remove(handler)
            if not handlerList:
                del self.__handlers[event_type]
        except KeyError:
            pass

    def publish_event(self, event):
        """发送事件，向事件队列中存入事件"""
        self.__eventQueue.put(event)

class Event(object):
    # 事件类

    def __init__(self, event_type, **kwargs):
        self.__dict__ = kwargs
        self.event_type = event_type

    def __repr__(self):
        return ' '.join('{}:{}'.format(k, v) for k, v in self.__dict__.items())

class EVENT(Enum):
    # 系统初始化后触发

    On_Init = 'on_init'

    On_Bar = 'on_bar'

    On_Tick = 'on_tick'

    On_Order = 'on_order'

    On_Trade = 'on_trade'

    On_Scheduler = 'on_scheduler'

    On_Open = 'on_open'

    On_Close = 'on_close'


# -*- coding: utf-8 -*- 

from datetime import  *  
from enum import IntEnum
from iqsopenapi.models.Order import *
from iqsopenapi.models.Contract import *

class Order(object):
    """委托请求"""

    def __init__(self):
        """订单ID"""

        """订单ID"""
        self.order_id = ""

        """策略编号"""
        self.strategy_id = ""

        """账户编号"""
        self.account_id = ""

        """股票代码或合约代码"""
        self.symbol = ""

        """交易所"""
        self.exchange = Exchange.UnKnow

        """委托方向"""
        self.side = OrderSide.Buy

        """委托价格"""
        self.price = 0.0

        """开仓还是平仓 (期货中使用) 非期货为None"""
        self.offset = Offset.UnKnow

        """委托数量"""
        self.quantity = 0   

        """委托时间"""
        self.order_time = datetime.now()

        """委托类型"""
        self.order_type = OrderType.LMT

        """委托状态"""
        self.status = OrderStatus.UnKnow

        """成交数量"""
        self.filled = 0

        """备注，失败时包含失败信息"""
        self.note = ""

    def isopen(self):
        """是否打开的订单"""
        return self.status != OrderStatus.Cancelled and self.status != OrderStatus.Filled and self.status != OrderStatus.Rejected;
     
class OrderType(IntEnum):
        """委托类型"""

        """限价"""
        LMT = 0
        """市价"""
        MKT = 1

class OrderStatus(IntEnum):
        """ 未知""" 
        UnKnow = -1
        """ 未发（下单指令还未发送到下游）"""
        NotSent = 0
        """ 1 已发（下单指令已发送给下游）"""
        Sended = 1
        """ 2 已报（下单指令已报给交易所）"""
        Accepted = 2
        """ 部分成交 """
        PartiallyFilled = 3
        """ 4 已撤（可能已经部分成交，要看看filled字段）"""
        Cancelled = 4
        """ 5 全部成交 """
        Filled = 5
        """ 6 已拒绝 """
        Rejected = 6
        """ 7 撤单请求已发送，但不确定当前状态 """
        PendingCancel = 7

class OrderSide(IntEnum):
        """买卖方向"""

        """买入"""
        Buy = ord('B')
        """卖出"""
        Sell = ord('S')

class Offset(IntEnum):
        """未知"""
        UnKnow = 0
        """开仓""" 
        Open = 1
        """平仓 """ 
        Close = 2
        """ 平今 """
        CloseToday = 3
        """ 平昨 """
        CloseYesterday = 4


# -*- coding: utf-8 -*-
from iqsopenapi.models.Order import *
from iqsopenapi.util.HttpUtil import *
from iqsopenapi.util.logutil import *
from iqsopenapi.util.DES3Cryptogram import *
from iqsopenapi.trade.realtrade.SupervisionHelper import *
from iqsopenapi.trade.realtrade.Models import *
from iqsopenapi.trade.realtrade.TradeEnums import *
from iqsopenapi.trade.realtrade.ParseManager import *
from iqsopenapi.util.Util import *
from urllib.parse import quote
import os
import string
from iqsopenapi.environment import *
import platform

class RealTradeApiCore:
    """期货交易接口"""

    def __init__(self):
        self.__supervision_helper = SupervisionHelper()
        self.__passwordKey = 'password'
        self.__tradeTokenKey = 'tradeToken'
        
        packType = 1028
        version = '2.4.5.2700'
        self.__appID = "YK_YKCJPC_2.0"

        self.__post_fix = f'?packType={packType}&version={version}&s=py'
        livetradeaddr = Environment.get_instance().get_livetradeaddr()
        self.__qryCounterUrl = self.__encodeUrl(f'{livetradeaddr}allbroker/qrycounterapi.ashx{self.__post_fix}')
        self.__testConnectUrl = self.__encodeUrl(f'gateway/system/testconnect.ashx{self.__post_fix}')
        self.__loginUrl = self.__encodeUrl(f'gateway/login.ashx{self.__post_fix}')
        self.__sendOrderUrl = self.__encodeUrl(f'gateway/sendorder.ashx{self.__post_fix}')
        self.__cancelOrderUrl = self.__encodeUrl(f'gateway/cancelorder.ashx{self.__post_fix}')
        self.__qryEntrustUrl = self.__encodeUrl(f'gateway/qryordertrade.ashx{self.__post_fix}')
        self.__fundUrl = self.__encodeUrl(f'gateway/qryasset.ashx{self.__post_fix}')
        self.__positionUrl = self.__encodeUrl(f'gateway/qryposition.ashx{self.__post_fix}')
        self.__doneUrl = self.__encodeUrl(f'gateway/qrytrade.ashx{self.__post_fix}')

    def GetCounterInfo(self, borkerType):
        """获取指定公司的柜台信息"""
        if(not borkerType): raise Exception("未设置柜台地址!")

        url = self.__qryCounterUrl
        kvp = [kv('brokerType', borkerType)]
        param = self.__getJson(kvp)
        self.__wirteInfo(f'GetCounterInfo Url:{url}')
        info = self.__post_crypt_core(url, param, ParseTradeCounterModel)
        if info.ErrorNo != 0:
            self.__wirteError(f'GetCounterInfo, {info.Json}')
            return None
        else: 
            self.__wirteInfo(f'GetCounterInfo, {info.Json}')
            return info
        pass

    def GetCounterApi(self, counter):
        '''自动获取最优的柜台'''
        def pingTime(x):
            ping, result = ExecutedTime(lambda :self.__testConnect(x.ApiAddr))
            if not result: ping = -1
            x.Ping = ping
            return x.Ping > 0
        if(not counter.ApiList or len(counter.ApiList) == 0):raise Exception("获取不到柜台地址!")
        counterApis = list(filter(pingTime, counter.ApiList))
        counterApi = Minimum(counterApis, lambda x:x.Ping)
        return counterApi
        
    def AccountLogin(self, account):
        """登录"""
        borkerType = account.BrokerType
        counterId = account.CounterID
        addr = account.CounterAddr
        userName = account.AccountID
        password = account.PassWord
        super_info = account.SupInfo
        api_id = account.ApiInfoId

        if(not addr): raise Exception("未设置柜台地址!")
        if(super_info.TradeCounter == TradeCounterType.NONE): raise Exception("未设置柜台类型!")

        url = f'{addr}{self.__loginUrl}'
        kvp = [
            kv('brokerType', borkerType), 
            kv('accountID', userName), 
            kv(self.__passwordKey, password), 
            kv('apiInfoId', api_id), 
            kv('AppID', self.__appID), 
            kv('counterID', counterId)]
        js = self.__supervision_helper.SetInfo(super_info)
        kvp[len(kvp):len(js)] = js
        param = self.__getJson(kvp)
        self.__wirteInfo(f'AccountLogin Url:{url}')
        info = self.__post_crypt_core(url, param, ParseLoginResult)
        if info.ErrorNo != 0:
            self.__wirteError(f'AccountLogin, {info.Json}')
            return None
        else: 
            self.__wirteInfo(f'AccountLogin, {info.Json}')
            return info

    def SendOrder(self, order):
        """下单"""
        account = self.__getUserInfo()
        url = f'{account.CounterAddr}{self.__sendOrderUrl}'
        kvp = [
            kv(self.__tradeTokenKey, account.TradeToken), 
            kv(self.__passwordKey, account.PassWord), 
            kv('symbol', order.symbol), 
            kv('exchange', order.exchange.name), 
            kv('orderSide', order.side.name), 
            kv('price', order.price), 
            kv('quantity', order.quantity), 
            kv('orderType', order.order_type.value), 
            kv('offset', order.offset.value)]
        param = self.__getJson(kvp)
        self.__wirteInfo(f'SendOrder Url:{url}')
        model = self.__post_crypt_core(url, param, ParseSendOrderModel)
        self.__wirteInfo(f'SendOrder, {model.Json}')
        return model

    def CancelOrder(self, orderID):
        """撤单"""
        account = self.__getUserInfo()
        url = f'{account.CounterAddr}{self.__cancelOrderUrl}'
        kvp = [
            kv(self.__tradeTokenKey, account.TradeToken), 
            kv(self.__passwordKey, account.PassWord), 
            kv('orderID', orderID)]
        param = self.__getJson(kvp)
        self.__wirteInfo(f'CancelOrder Url:{url}')
        model = self.__post_crypt_core(url, param, ParseBaseModel)
        self.__wirteInfo(f'CancelOrder, {model.Json}')
        return model

    def GetAccountFund(self):
        """查询资金"""
        account = self.__getUserInfo()
        url = f'{account.CounterAddr}{self.__fundUrl}'
        kvp = [
            kv(self.__tradeTokenKey, account.TradeToken), 
            kv(self.__passwordKey, account.PassWord)]
        param = self.__getJson(kvp)
        self.__wirteInfo(f'GetAssetInfo Url:{url}')
        model = self.__post_crypt_core(url, param, ParseUserAssetsModel)
        if model.ErrorNo != 0:
            self.__wirteError(f'GetAssetInfo, {model.Json}')
            return None
        else:
            self.__wirteInfo(f'GetAssetInfo, {model.Json}')
            return model

    def GetEntrustInfos(self, id=None):
        """查询委托"""
        account = self.__getUserInfo()
        url = f'{account.CounterAddr}{self.__qryEntrustUrl}'
        kvp = [kv(self.__tradeTokenKey, account.TradeToken)]
        if id != None:
            kvp[len(kvp):1] = [kv('orderID', id)]
        param = self.__getJson(kvp)
        self.__wirteInfo(f'GetEntrustInfos Url:{url}')
        info = self.__post_crypt_core(url, param, ParseEntrustModel)
        if info.ErrorNo != 0:
            self.__wirteError(f'GetEntrustInfos, {info.Json}')
            return None
        else: 
            self.__wirteInfo(f'GetEntrustInfos, {info.Json}')
            return info.EntrustInfos

    def GetPositionInfos(self):
        """查询持仓"""
        account = self.__getUserInfo()
        url = f'{account.CounterAddr}{self.__positionUrl}'
        kvp = [
            kv(self.__tradeTokenKey, account.TradeToken), 
            kv(self.__passwordKey, account.PassWord), 
            kv('qryFund', 1)]
        param = self.__getJson(kvp)
        self.__wirteInfo(f'GetPositions Url:{url}')
        info = self.__post_crypt_core(url, param, ParsePositionModel)
        if info.ErrorNo != 0:
            self.__wirteError(f'GetPositions, {info.Json}')
            return None
        else: 
            self.__wirteInfo(f'GetPositions, {info.Json}')
            return info.Positions

    def GetTrades(self):
        """获取当日成交"""
        account = self.__getUserInfo()
        url = f'{account.CounterAddr}{self.__doneUrl}'
        kvp = [
            kv(self.__tradeTokenKey, account.TradeToken), 
            kv(self.__passwordKey, account.PassWord)]
        param = self.__getJson(kvp)
        self.__wirteInfo(f'GetTrades Url:{url}')
        info = self.__post_crypt_core(url, param, ParseDoneModel)
        if info.ErrorNo != 0:
            self.__wirteError(f'GetTrades, {info.Json}')
            return None
        else: 
            self.__wirteInfo(f'GetTrades, {info.Json}')
            return info.Dones

    def SetUserInfo(self, user):
        self.__user_info = user

    def __testConnect(self, addr):
        url = f"{addr}{self.__testConnectUrl}?&apiAddr={addr}"
        result = httpGet(url)
        js = json.loads(result)
        if("Connected" == js['status']): return js
        else: 
            self.__wirteError(f'TestConnect, {result}')
            return None

    def __encodeUrl(self, url):
        return quote(url, safe=string.printable)

    def __getJson(self, kvp):
        return '{' + ','.join(['"%s":"%s"' % (item.Key, item.Value)  for item in kvp]) + '}'

    def __post_crypt_core(self, url, param, parsecore):
        def parse(x):
            de = decrypt(x)
            return parsecore(json.loads(de))
        en = f'param={encrypt(param)}'
        return parse(httpFormPost(url, en))

    def __wirteError(self, error):
        '''写错误日志'''
        logger.error(error)

    def __wirteInfo(self, info):
        '''写日志'''
        logger.info(info)

    def __getUserInfo(self):
        if(self.__user_info != None): return self.__user_info;
        else: raise Exception("未登录")

if __name__ == '__main__':
    info = { 
    'strategy_id':'1021',
    'account':'10111131',
    'password':'864201',
    'broker_type':1,
    'comp_counter':1,
    }
    brokerType = info['broker_type']
    counterId = info['comp_counter']
    user = UserInfo()
    user.AccountID = info['account']
    user.PassWord = info['password']
    user.BrokerType = brokerType
    user.CounterID = counterId

    api = RealTradeApiCore()
    '''
    counterInfo = api.GetCounterInfo(brokerType)
    counters = list(filter(lambda x:x.CounterId == counterId, counterInfo.CounterList))
    if(not counters or len(counters) == 0):raise Exception("没有对应的柜台id")
    counter = counters[0]
    user.SupInfo.TradeCounter = counter.TradeCounter
    counterApi = api.GetCounterApi(counter)

    user.ApiInfoId = counterApi.ApiInfoId
    user.CounterAddr = counterApi.ApiAddr
    login = api.AccountLogin(user)
    user.TradeToken = login.TradeToken
    '''

    user.SupInfo.TradeCounter = TradeCounterType.HS
    user.ApiInfoId = 19
    user.CounterAddr = 'http://222.178.95.209:11000/'
    user.TradeToken = '0d81ca3e14d84d51a8a7848c8cfbe16b'

    api.SetUserInfo(user)

    #positions = api.GetPositionInfos()
    #entrust = api.GetEntrustInfos()
    #funds = api.GetAccountFund()

    order = Order()
    order.Symbol = 'c2101'
    order.Exchange = Exchange.DCE
    order.OrderSide = OrderSide.Buy
    order.Offset = Offset.Open
    order.OrderType = OrderType.LMT
    order.Price = 1928
    order.Quantity = 1

    if(api.SendOrder(order)): pass

    if(api.CancelOrder(order.OrderId)): pass

    aa = api



# -*- coding: utf-8 -*-
from iqsopenapi.models import *
from iqsopenapi.util import *
from iqsopenapi.trade import *
from iqsopenapi.trade.papertrade.message import *
from iqsopenapi.core import *
from iqsopenapi.environment import *
import json
import time
import uuid
import datetime
import traceback
import threading
from copy import copy

class PaperTradeApi(ITradeApi):
    """期货模拟交易"""

    def __init__(self,event_bus):
        """构造函数"""
        super(PaperTradeApi,self).__init__(event_bus)

        self.__event_bus = event_bus

        self.__strategy_id = Environment.get_instance().config.acct_info['strategy_id']
        self.__account_id = Environment.get_instance().config.acct_info['account']

        cfg = websocketCfg()
        cfg.address = Environment.get_instance().get_trademsgaddr()
        cfg.hbContent = json.dumps({ 'requestType' : 'HeartBeat','reqID' : '' })
        cfg.recvCallback = self.__OnRecv
        cfg.connectedCallback = self.__AuotSubscribe
        self.__wsClient = WebSocketClient(cfg)
        self.__subscribes = []
        self.__responseMsgCache = MemCache()

    def Init(self):
        """初始化"""
        if not self.__wsClient.Connect():
            logger.error("paper trade connect failed!")
            return False
        self.__wsClient.AutoHeartBeat()
        if not self.__Subscribe():
            logger.error("paper trade subscribe failed!")
            return False
        return True

    def __AuotSubscribe(self):
        """自动订阅"""
        try:
            if len(self.__subscribes) > 0:
                logger.debug("连接成功，开始订阅...")
                subRet = self.Subscribe(*self.__subscribes)
                logger.debug("连接成功，订阅完成：{0}".format(subRet))
        except Exception as e:
            logger.exception(e)

    def __Subscribe(self):
        id = uuid.uuid1().hex
        req = {"RequestType":RequestType.Subscribe,"ReqID":id,"StrategyId":self.__strategy_id,"TradeAccount":self.__account_id}
        message = json.dumps(req,ensure_ascii=False)
        if not self.__Send(id,message):
            logger.error("Subscribe papertrade send fail:{0}".format(message))
            return False
        logger.info("Subscribe papertrade succeed:{0}".format(message))
        return True

    def __Send(self,id,msg):
        """消息发送"""
        if not msg:
            return False
        if not self.__wsClient.Send(msg):
            logger.error("send fail:{0}".format(msg))
            return False
        i = 0
        while i < 60:
            reply = self.__responseMsgCache.get(id)
            if reply:
                self.__responseMsgCache.delete(id)
                return reply["ErrorNo"] == 0
            i += 1
            time.sleep(0.1)
        return False

    def __OnRecv(self,msg):
        '''收到消息'''
        if not msg:
            return
        responseType = msg.get("rt")
        datas = msg.get("data")
        if responseType == ResponseType.Reply.name:
            self.__responseMsgCache.set(msg.get("ReqID"), msg, 60)
            return
        if responseType == ResponseType.OrderChange.name:
            for item in datas:
                order = self.__ToOrder(item)
                if not order:
                    return
                order_event = Event(EVENT.On_Order,data = order)
                self.__event_bus.publish_event(order_event)
        if responseType == ResponseType.ExecutionReport.name:
            for item in datas:
                trade = self.__ToTrade(item)
                if not trade:
                    return
                trade_event = Event(EVENT.On_Trade,data = trade)
                self.__event_bus.publish_event(trade_event)

    def SendOrder(self,symbol, exchange, orderSide, price, quantity, orderType, offset):
        """下单"""
        if not symbol or symbol == '':
            logger.error("symbol paramter is empty")
            return None
        if not isinstance(quantity, int):
            logger.error("quantity paramter is not int")
            return None

        order = Order()
        order.exchange = exchange
        order.filled = 0
        order.note = ''
        order.offset = offset
        order.order_id = ''
        order.side = orderSide
        order.order_time = datetime.datetime.now()
        order.order_type = orderType
        order.price = price
        order.quantity = quantity
        order.status = OrderStatus.NotSent
        order.symbol = symbol
        order.strategy_id = self.__strategy_id
        order.account_id = self.__account_id
        order_event = Event(EVENT.On_Order,data = order)
        self.__event_bus.publish_event(order_event)

        params = {'symbol':symbol,'exchange':int(exchange),'orderSide':int(orderSide),'price':price,'quantity':quantity,'orderType':int(orderType),'offset':int(offset),'strategyId':self.__strategy_id,'tradeAccount':self.__account_id}
        logger.info("begin sendorder，param:{0}".format(json.dumps(params,ensure_ascii=False)))

        url = Environment.get_instance().get_apiurl('api/PaperTrade/SendOrder')
        resp = httpJsonPost(url, params)
        logger.info("sendorder complated，param:{0},resp:{1}".format(json.dumps(params,ensure_ascii=False),resp))
        if not resp:
            logger.error("response is empty:{0},{1}".format(url,json.dumps(params,ensure_ascii=False)))
            return None
        js = json.loads(resp,encoding='utf-8')

        neworder = copy(order)
        if js.get('error_no') != 0:
            logger.error("sendorder failed:{0},{1},{2}".format(url,json.dumps(params,ensure_ascii=False),resp))
            neworder.status = OrderStatus.Rejected
            neworder.note = js.get('error_info')
        else:
            orderId = js.get('data')['orderId']
            neworder.order_id = str(orderId)
            neworder.status = OrderStatus.Sended
        neworder_event = Event(EVENT.On_Order,data = neworder)
        self.__event_bus.publish_event(neworder_event)
        return neworder

    def CancelOrder(self,order):
        """撤单"""
        if not order or not order.order_id:
            logger.error("invalid order can't cancel")
            return None

        cancelOrder = copy(order)
        cancelOrder.status = OrderStatus.PendingCancel
        cancelOrder.strategy_id = self.__strategy_id
        cancelOrder.account_id = self.__account_id
        order_event = Event(EVENT.On_Order,data = cancelOrder)
        self.__event_bus.publish_event(order_event)

        params = {'orderId':order.order_id,'strategyId':self.__strategy_id,'tradeAccount':self.__account_id}
        logger.info("begin cancelorder，param:{0}".format(json.dumps(params,ensure_ascii=False)))

        url = Environment.get_instance().get_apiurl('api/PaperTrade/CancelOrder')
        resp = httpJsonPost(url,params)
        if not resp:
            logger.error("response is empty:{0},{1}".format(url,json.dumps(params,ensure_ascii=False)))
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("cancel order failed:{0},{1},{2}".format(url,json.dumps(params,ensure_ascii=False),resp))

        newCancelOrder = self.GetOrder(order.order_id)

        newCancelOrder_event = Event(EVENT.On_Order,data = newCancelOrder)
        self.__event_bus.publish_event(newCancelOrder_event)

        return newCancelOrder

    def GetAccount(self):
        """获取资产信息"""

        url = Environment.get_instance().get_apiurl('api/PaperTrade/GetAccount')
        params = {'strategyId':self.__strategy_id,'tradeAccount':self.__account_id}
        resp = httpJsonPost(url,params)
        if not resp:
            logger.error("response is empty:{0},{1}".format(url,json.dumps(params,ensure_ascii=False)))
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("failed:{0},{1},{2}".format(url,json.dumps(params,ensure_ascii=False),resp))
            return None
        resData = js['data']
        asset = self.__ToAccount(resData)
        return asset

    def GetOrder(self,orderId):
        """根据订单ID号 获取订单详情"""
        orders = self.GetOrders()
        if not orders:
           return None
        for x in orders:
            if x.order_id == orderId:
                return x
        return None

    def GetOpenOrders(self):
        """获取打开的订单"""
        orders = self.GetOrders()
        if not orders:
           return None
        openOrders = []
        for x in orders:
            if x.isopen():
                openOrders.append(x)
        return openOrders

    def GetOrders(self):
        """获取委托"""

        url = Environment.get_instance().get_apiurl('api/PaperTrade/GetOrders')
        params = {'strategyId':self.__strategy_id,'tradeAccount':self.__account_id}
        resp = httpJsonPost(url,params)
        if not resp:
            logger.error("response is empty:{0},{1}".format(url,json.dumps(params,ensure_ascii=False)))
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("failed:{0},{1},{2}".format(url,json.dumps(params,ensure_ascii=False),resp))
            return None
        resData = js.get('data')
        if not resData:
            return []
        orders = []
        for item in resData:
            order = self.__ToOrder(item)
            orders.append(order)
        return orders

    def GetPositions(self):
        """获取持仓"""

        url = Environment.get_instance().get_apiurl('api/PaperTrade/GetPositions')
        params = {'strategyId':self.__strategy_id,'tradeAccount':self.__account_id}
        resp = httpJsonPost(url,params)
        if not resp:
            logger.error("response is empty:{0},{1}".format(url,json.dumps(params,ensure_ascii=False)))
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("failed:{0},{1},{2}".format(url,json.dumps(params,ensure_ascii=False),resp))
            return None
        resData = js.get('data')
        if not resData:
            return []
        posList = []
        for item in resData:
            pos = self.__ToPos(item)
            posList.append(pos)
        return posList

    def GetTrades(self):
        """获取委托"""

        url = Environment.get_instance().get_apiurl('api/PaperTrade/GetExecutions')
        params = {'strategyId':self.__strategy_id,'tradeAccount':self.__account_id}
        resp = httpJsonPost(url,params)
        if not resp:
            logger.error("response is empty:{0},{1}".format(url,json.dumps(params,ensure_ascii=False)))
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("failed:{0},{1},{2}".format(url,json.dumps(params,ensure_ascii=False),resp))
            return None
        resData = js.get('data')
        if not resData:
            return []
        trades = []
        for item in resData:
            trade = self.__ToTrade(item)
            trades.append(trade)
        return trades

    def __ToOrder(self, item):
        order = Order()
        order.exchange = Exchange(item.get('e'))
        order.filled = item['f']
        order.note = item['n']
        order.offset = Offset(item['oft'])
        order.order_id = item['oid']
        order.side = OrderSide(item['osd'])
        order.order_time = datetime.datetime.strptime(item['ote'],'%Y-%m-%d %H:%M:%S')
        order.order_type = OrderType(item['otp'])
        order.price = item['p']
        order.quantity = item['q']
        order.status = OrderStatus(item['ost'])
        order.symbol = item['s']
        order.strategy_id = self.__strategy_id
        order.account_id = self.__account_id
        return order

    def __ToTrade(self, item):
        trade = Trade()
        trade.strategy_id = self.__strategy_id
        trade.account_id = self.__account_id
        trade.exchange = Exchange(item.get('e'))
        trade.symbol = item['s']
        trade.order_id = item['oid']
        trade.trade_id = item['eid']
        trade.side = OrderSide(item['osd'])
        trade.price = item['p']
        trade.offset = Offset(item['oft'])
        trade.quantity = item['q']
        trade.filled_time = datetime.datetime.strptime(item['ft'],'%Y-%m-%d %H:%M:%S')
        return trade

    def __ToPos(self, item):
        pos = Position()
        pos.strategy_id = self.__strategy_id
        pos.account_id = self.__account_id
        pos.symbol = item['s']
        pos.exchange = Exchange(item['e'])
        pos.last_px = item['lp']
        pos.quantity = item['q']
        pos.frozen = item['fz']
        pos.cost = item['c']
        pos.side = PosSide(item['ps'])
        pos.margin = item['m']
        pos.value = item['v']
        pos.today_qty = item['tq']
        pos.today_avl =  item['ta']
        pos.profit = item['pf']
        return pos

    def __ToAccount(self, resData):
        account = Account()
        account.strategy_id = self.__strategy_id
        account.account_id = self.__account_id
        account.total_value = resData['tv']
        account.available = resData['avl']
        account.frozen_cash = resData['fc']
        account.market_value = resData['mv']
        account.margin = resData['m']
        account.begin_balance = resData['bb']
        account.withdraw = resData['wd']
        account.currency = Currency.CNY
        return account




# -*- coding: utf-8 -*-
from enum import IntEnum
from iqsopenapi.models.Contract import *

class Account(object):
    """资产信息"""

    def __init__(self):
        
        """策略编号"""
        self.strategy_id = ""

        """账户编号"""
        self.account_id = ""

        """总权益"""
        self.total_value = 0.0

        """可用资金"""
        self.available = 0.0

        """冻结资金"""
        self.frozen_cash = 0.0

        """总市值"""
        self.market_value = 0.0

        """保证金"""
        self.margin = 0.0

        """期初权益"""
        self.begin_balance = 0.0

        """可提资金"""
        self.withdraw = 0.0

        """币种"""
        self.currency = Currency.CNY





# -*- coding: utf-8 -*- 
from iqsopenapi.marketdata.message import *

class HeartBeatReq(RequestMessage):
    """心跳请求"""

    def __init__(self):
        """请求类型"""
        self.RequestType = RequestType.HeartBeat



# -*- coding: utf-8 -*-
from iqsopenapi.marketdata.message.IMessage import *
from iqsopenapi.marketdata.message.HeartBeatReq import *
from iqsopenapi.marketdata.message.ReplyResp import *
from iqsopenapi.marketdata.message.SubscribeReq import *
from iqsopenapi.marketdata.message.UnsubscribeReq import *



# -*- coding: utf-8 -*-
class IBasicDataApi(object):
    """基础数据接口"""

    def __init__(self):
        """构造函数"""
        pass

    def GetContract(self, symbol, exchange):
        """获取合约信息"""
        pass

    def GetMainContract(self, variety):
        """获取主力合约信息（期货）"""
        pass

    def GetOpenTimes(self,begin,end):
        """获取开盘时间"""
        pass




# -*- coding: utf-8 -*-
from iqsopenapi.marketdata.message import *

class SubscribeReq(RequestMessage):
    """订阅请求"""
    def __init__(self):
        """请求类型"""
        self.RequestType = RequestType.Subscribe

        """订阅列表"""
        self.Subscribes = []




# -*- coding: utf-8 -*-
from iqsopenapi.util.HttpUtil import *
from iqsopenapi.util.logutil import *
from iqsopenapi.basicdata.IBasicDataApi import *
from iqsopenapi.models.Contract import *
from iqsopenapi.environment import *
import json
import datetime

class BasicDataApi(IBasicDataApi):
    """期货数据API"""

    def __init__(self):
        """构造函数"""
        super(BasicDataApi,self).__init__()

    def GetContract(self, symbol, exchange):
        """获取合约信息"""
        url = Environment.get_instance().get_apiurl('api/BasicData/GetContract')
        params = {'symbol':symbol,'exchange':exchange}
        strParams = json.dumps(params,ensure_ascii=False)
        resp = httpJsonPost(url,params)
        if not resp:
            logger.error("请求应答为空：" + url + "," + strParams)
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("request:" + url + "," + strParams + "response：" + resp)
            return None
        data = js.get('data')
        if not data:
            logger.error("data为空,request:" + url + "," + strParams + "response：" + resp)
            return None
        contract = self.__ToContract(data)
        return contract

    def GetMainContract(self, variety):
        """获取主力合约信息（期货）"""
        url = Environment.get_instance().get_apiurl('api/BasicData/GetMainContract')
        params = {'varietyCode':variety}
        strParams = json.dumps(params,ensure_ascii=False)
        resp = httpJsonPost(url,params)
        if not resp:
            logger.error("请求应答为空：" + url + "," + strParams)
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("request:" + url + "," + strParams + "response：" + resp)
            return None
        data = js.get('data')
        if not data:
            logger.error("data为空,request:" + url + "," + strParams + "response：" + resp)
            return None
        contract = self.__ToContract(data)
        return contract

    def GetOpenTimes(self,begin,end):
        """获取开盘时间"""
        url = Environment.get_instance().get_apiurl('api/BasicData/GetOpenTimes')
        params = {'begin':begin.strftime("%Y%m%d%H%M%S"),'end':end.strftime("%Y%m%d%H%M%S")}
        strParams = json.dumps(params,ensure_ascii=False)
        resp = httpJsonPost(url,params)
        if not resp:
            logger.error("请求应答为空：" + url + "," + strParams)
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("request:" + url + "," + strParams + "response：" + resp)
            return None
        data = js.get('data')
        if not data:
            return []
        open_times = []
        for x in data:
            trade_time = self.__ToTradeTime(x)   
            open_times.append(trade_time)
        return open_times

    def __ToTradeTime(self, data):
        time_range = tradeTime()
        time_range.begin = data['begin']
        time_range.end = data['end']
        return time_range

    def __ToContract(self,data):
        """根据clientOrderID 获取订单详情"""
        if not data:
            return None
        item = Contract()
        item.contract_name = data["contractName"]
        item.contract_type = ContractType(data["contractType"])
        item.exchange = Exchange(data["exchange"])
        item.expire_date = datetime.datetime.strptime(data["expiryDate"],'%Y-%m-%d')
        item.list_date = datetime.datetime.strptime(data["listingDate"],'%Y-%m-%d')
        item.lots = data["lots"]
        item.step = data["priceStep"]
        item.right = data["right"]
        item.strike_px = data["strikePx"]
        item.symbol = data["symbol"]
        for x in data['tradeTimes']:
            time_range = self.__ToTradeTime(x)
            item.trade_times.append(time_range)
        return item

if __name__ == '__main__':

    config = {
      "runtime": "DEBUG"
    }
    environment = Environment(config)

    api = BasicDataApi()
    val1 = api.GetContract("rb2005","SHFE")
    logger.info(val1)

    val2 = api.GetMainContract("rb")
    logger.info(val2)

    begin = datetime.datetime.now()
    end = begin + datetime.timedelta(days=1)
    val3 = api.GetOpenTimes(begin,end)
    
    pass


# -*- coding: utf-8 -*-

import codecs
import six
from iqsopenapi.core.IStrategyLoader import *

from iqsopenapi.util.logutil import *

def compile_strategy(source_code, strategy, scope):
    try:
        code = compile(source_code, strategy, 'exec')
        six.exec_(code, scope)
        return scope
    except Exception as e:
        logger.exception(e)

class FileStrategyLoader(IStrategyLoader):
    def __init__(self, strategy_file_path):
        self._strategy_file_path = strategy_file_path

    def load(self, scope):
        with codecs.open(self._strategy_file_path, encoding="utf-8") as f:
            source_code = f.read()

        return compile_strategy(source_code, self._strategy_file_path, scope)


class SourceCodeStrategyLoader(IStrategyLoader):
    def __init__(self, code):
        self._code = code

    def load(self, scope):
        return compile_strategy(self._code, "strategy.py", scope)


class UserFuncStrategyLoader(IStrategyLoader):
    def __init__(self, user_funcs):
        self._user_funcs = user_funcs

    def load(self, scope):
        for user_func in six.itervalues(self._user_funcs):
            user_func.__globals__.update(scope)
        scope.update(self._user_funcs)
        return self._user_funcs

if __name__ == '__main__':

    loader = FileStrategyLoader(r'D:\gitwork\inquantstudio\IQSOpenApi\IQS.OpenApi.Python\iqsopenapi\example\teststrategy.py')

    from iqsopenapi import *
    from copy import copy

    scope = copy(iqsopenapi.__dict__)

    loader.load(scope)

    pass


# -*- coding: utf-8 -*-
import datetime
import time
from iqsopenapi.trade.realtrade.Models import *
from iqsopenapi.models.Account import *
from iqsopenapi.models.Trade import *
from iqsopenapi.models.Order import *
from iqsopenapi.models.Position import *
from iqsopenapi.util.logutil import *

def __ParseBaseModelCore(model, json):
    if (model == None): model = ReposeModelBase()
    model.Json = json
    model.ErrorInfo = json['error_info']
    model.ErrorNo = json['error_no']
    return model

def ParseBaseModel(json):
    if not json: return None
    return __ParseBaseModelCore(None, json)

def ParseFuturesCompanysModel(json):
    if not json: return None
    model = FuturesCompanysModel()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    model.ArgeementTip = json['argeementTip']
    model.ArgeementHerfText = json['argeementHerfText']
    model.ArgeementHerf = json['argeementHerf']
    model.AppQRImage = json['appQRImg']
    data = json['data']
    for item in data:
        info = CompanyInfo()
        info.CompanyId = item['companyID']
        info.BrokerType = item['brokerType']
        info.CompanyName = item['brokerName']
        info.Description = item['desc']
        info.Phone = item['phone']
        info.Icon = item['ico']
        info.IsOnLine = item['isOnline']
        info.OpenAccountAddress = item['hkUrl']
        counter = item.get('tradingCounter')
        if (counter != None): info.TradeCounter = TradeCounterType(counter)
        model.CompanyInfos.append(info)
    return model

def ParseTradeCounterModel(json):
    if not json: return None
    model = TradeCounterModel()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    for item in json['counterList']:
        entity = TradeCounterEntity()
        entity.CounterId = item['counterID']
        entity.CounterName = item['counterName']
        counter = item.get('tradingCounter')
        if (counter != None): entity.TradeCounter = TradeCounterType(counter)
        for api in item['apiList']:
            api_entity = TradeApiEntity()
            api_entity.ApiAddr = api['apiAddr']
            api_entity.ApiName = api['apiName']
            api_entity.ApiInfoId = api['apiInfoId']
            api_entity.wsAddr = api['wsAddr']
            entity.ApiList.append(api_entity)
        model.CounterList.append(entity)
    return model

def ParseLoginResult(json):
    if not json: return None
    model = LoginResult()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    model.TradeToken = json['tradeToken']
    model.IsSigned = json['isSigned'] != 0
    return model

def ParseConOrderModel(json):
    if not json: return None
    model = ConOrderModel()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    if(not ('data' in json)): return model
    for item in json['data']:
        info = ConditionOrderEntity()
        info.ConditionId = item['conditionID']
        info.Symbol = item['symbol']
        info.ContractId = item['contractID']
        info.Status = item['status']
        info.StatusStr = item['statusStr']
        info.OrderSide = OrderSide(item['orderSide']);
        info.OrderSideStr = item['orderSideStr']
        info.Offset = Offset(item['offset'])
        info.OffSetStr = item['offsetStr']
        info.OrderType = OrderType(item['orderType'])
        info.OrderTypeStr = item['orderTypeStr']
        info.Price = item['price']
        info.Quantity = item['quantity']
        info.ExpireType = ConOrderTimeValidTypeEnum(item['expireType'])
        info.AddTimeStr = item['addTimeStr']
        info.ExpireTimeStr = item['expireTimeStr']
        for ckv in item['condiValues']:
            kvi = kv()
            kvi.Key = ckv['k']
            kvi.Value = ckv['v']
            info.CondiValues.append(kvi)
        info.CondiValueStr = item['condiValueStr']
        info.OrderNo = item['orderNo']
        info.OrderRet = item['orderRet']
        info.OrderNote = item['orderNote']
        info.TriggerTimeStr = item['triggerTimeStr']
        model.Data.append(info)
    return model

def ParseStopOrderNotTriggerModel(json):
    if not json: return None
    model = StopOrderNotTriggerModel()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    if(not ('data' in json)): return model
    for item in json['data']:
        info = StopOrderNotTrigger()
        info.StopId = item['stopID']
        info.Symbol = item['symbol']
        info.Name = item['name']
        info.PosSide = PosSide(item['posSide'])
        info.PosSideStr = item['posSideStr']
        info.StopType = item['stopType']
        info.StopTypeStr = item['stopTypeStr']
        info.TriggerPrice = item['stopPrice']
        info.OrderType = item['orderType']
        info.OrderTypeStr = item['orderTypeStr']
        info.Quantity = item['quantity']
        info.StatusStr = item['statusStr']
        model.Data.append(info)
    return model

def ParseUserAssetsModel(json):
    if not json: return None
    model = UserAssetsModel()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    info = Account()
    info.account_id = json['accountID']
    info.total_value = float(json['total'])
    info.available = float(json['available'])
    info.frozen_cash = float(json['frozenBalance'])
    info.market_value = float(json['marketBalance'])
    info.margin = float(json['margin'])
    info.begin_balance = float(json['beginBalance'])
    info.withdraw = float(json['withdraw'])
    info.currency = Currency[json['currency']]
    model.AssetInfo = info
    return model

def ParseEntrustModel(json):
    if not json: return None
    model = EntrustModel()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    for item in json['data']:
        info = Order()
        info.order_id = item['orderID']
        info.symbol = item['symbol']
        info.exchange = Exchange(item['exchange'])
        info.side = OrderSide[item['orderSide']]
        info.price = float(item['price'])
        info.offset = Offset(item['offset'])
        info.quantity = int(item['quantity'])
        info.order_time = datetime.strptime(item['orderTime'],'%Y%m%d%H%M%S')
        #委托类型 写死限价
        info.order_type = OrderType.LMT
        info.status = OrderStatus(item['orderStatus'])
        info.filled = int(item['filled'])
        info.note = item['errorMsg']
        model.EntrustInfos.append(info)
    return model

def ParsePositionModel(json):
    if not json: return None
    model = PositionModel()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    if(not ('data' in json)): return model
    for item in json['data']:
        info = Position()
        info.symbol = item['symbol']
        info.exchange = Exchange(item['exchange'])
        info.last_px = float(item['lastPx'])
        quantity = int(item['volume'])
        info.quantity = quantity
        info.frozen = quantity - int(item['canCloseVol'])
        info.cost = float(item['positionCost'])
        info.side = PosSide(item['posSide'])
        info.margin = float(item['margin'])
        info.value = float(item['marketValue'])
        info.profit = float(item['dropIncome'])
        info.today_qty = int(item['todayVol'])
        info.today_avl = int(item['todayEnableVol'])
        model.Positions.append(info)
    return model

def ParseDoneModel(json):
    if not json: return None
    model = DoneModel()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    if(not ('data' in json)): return model
    for item in json['data']:
        info = Trade()
        info.trade_id = item['executionID']
        if (not ('orderID' in item)): logger.info('成交接口没有orderID!')
        else: info.order_id = item['orderID']
        info.symbol = item['symbol']
        info.exchange = Exchange(item['exchange'])
        info.side = OrderSide[item['orderSide']]
        info.price = float(item['price'])
        info.offset = Offset(item['offset'])
        info.quantity = quantity = int(item['volume'])
        info.filled_time = datetime.strptime(item['executionTime'],'%Y%m%d%H%M%S')
        model.Dones.append(info)
    return model

def ParseSendOrderModel(json):
    if not json: return None
    model = SendOrderModel()
    ret = __ParseBaseModelCore(model, json)
    if (ret.ErrorNo != 0): return model
    model.OrderID = json['orderID']
    return model


# -*- coding: utf-8 -*-

from iqsopenapi import *
import json

code = """
from iqsopenapi import *

def on_init(ctx):
    logger.info("init")
    ctx.subscribe(('rb2010.SHFE','TICK'),('IH2005.CFFEX','3m'))

def on_tick(ctx,tick):
    logger.info('tick:{0},{1},{2}'.format(tick.symbol,tick.local_time,tick.last))
"""

config = """
{
    "runtime":"DEBUG",
    "run_type":"PAPER_TRADING",
    "acct_info":{
            "strategy_id":"21",
            "account":"10030"
        }
}
"""

js = json.loads(config,encoding='utf-8')
run_code(code,js)


# -*- coding: utf-8 -*- 

from enum import IntEnum
from iqsopenapi.models.Contract import *

class Position(object):
    """持仓信息"""

    def __init__(self):
        
        
        """策略编号"""
        self.strategy_id = ""

        """账户编号"""
        self.account_id = ""

        """股票代码或合约代码"""
        self.symbol = ""

        """交易所"""
        self.exchange = Exchange.UnKnow

        """最新行情价"""
        self.last_px = 0.0

        """持仓数量"""
        self.quantity = 0

        """冻结数量"""
        self.frozen = 0

        """持仓成本"""
        self.cost = 0.0

        """持仓方向"""
        self.side = PosSide.Net

        """保证金"""
        self.margin = 0.0

        """市值"""
        self.value = 0.0

        """浮动盈亏"""
        self.profit = 0.0

        """今仓"""
        self.today_qty = 0.0

        """今仓可用"""
        self.today_avl = 0.0

class PosSide(IntEnum):
        """持仓方向"""

        """净持仓"""
        Net = 0
        """多仓"""
        Long = 1
        """空仓"""
        Short = 2


# -*- coding: utf-8 -*-
from iqsopenapi.marketdata.message import *

class UnsubscribeReq(RequestMessage):

    def __init__(self):
        """请求类型"""
        self.RequestType = RequestType.Unsubscribe

        """取消订阅列表"""
        self.Unsubscribes = []


# -*- coding: utf-8 -*-
from iqsopenapi.models import *
from iqsopenapi.util import *
from iqsopenapi.marketdata.message import *
from iqsopenapi.marketdata.IMarketApi import *
from iqsopenapi.core import *
from iqsopenapi.environment import *
import json
import time
import uuid
import iqsopenapi.util.logutil
import datetime
import traceback
import threading


class MarketApi(IMarketApi):
    """行情接口"""

    def __init__(self,event_bus):
        """构造函数"""
        super(MarketApi,self).__init__(event_bus)

        self.__event_bus = event_bus

        cfg = websocketCfg()
        cfg.address = Environment.get_instance().get_quoteaddr()
        cfg.hbContent = json.dumps({ 'requestType' : 'HeartBeat','reqID' : '' })
        cfg.recvCallback = self.__OnRecv
        cfg.connectedCallback = self.__AuotSubscribe
        self.__wsClient = WebSocketClient(cfg)
        self.__subscribes = []
        self.__responseMsgCache = MemCache()

    def Init(self):
        """初始化"""
        if not self.__wsClient.Connect():
            logger.error("market connect failed!")
            return False
        self.__wsClient.AutoHeartBeat()
        return True

    def __AuotSubscribe(self):
        """自动订阅"""
        try:
            if len(self.__subscribes) > 0:
                logger.debug("连接成功，开始订阅...")
                subRet = self.Subscribe(*self.__subscribes)
                logger.debug("连接成功，订阅完成：{0}".format(subRet))
        except Exception as e:
            logger.exception(e)

    def __OnRecv(self,msg):
        '''收到行情消息'''

        if not msg:
            return
        responseType = msg.get("rt")
        datas = msg.get("data")
        if responseType == ResponseType.Reply.name:
            self.__responseMsgCache.set(msg.get("ReqID"), msg, 60)
            return
        if responseType == ResponseType.TickData.name:
            for item in datas:
                tick = self.__ToTickData(item)
                if not tick:
                    return
                tick_event = Event(EVENT.On_Tick,data = tick)
                self.__event_bus.publish_event(tick_event)
        if responseType == ResponseType.KlineData.name:
            for item in datas:
                bar = self.__ToBar(item)
                if not bar:
                    return
                bar_event = Event(EVENT.On_Bar,data = bar)
                self.__event_bus.publish_event(bar_event)

    def GetSubscibes(self):
        """获取订阅列表"""
        return self.__subscribes

    def Subscribe(self,*subInfos):
        """行情订阅"""
        if not subInfos:
            logger.error('订阅参数不能为空')
            return False
        if not self.__wsClient.IsAvaliable():
            logger.error('行情未连接')
            return False
        req = SubscribeReq()
        req.ReqID = uuid.uuid1().hex
        req.RequestType = RequestType.Subscribe
        req.Subscribes = subInfos
        message = json.dumps(req.__dict__,ensure_ascii=False)
        if not self.__Send(req.ReqID,message):
            logger.error("Subscribe market send fail:{0}".format(message))
            return False
        logger.info("Subscribe market succeed:{0}".format(message))
        for x in subInfos:
            if x not in self.__subscribes:
                self.__subscribes.append(x)
        return True

    def __Send(self,id,msg):
        """消息发送"""
        if not msg:
            return False
        if not self.__wsClient.Send(msg):
            logger.error("send fail:{0}".format(msg))
            return False
        i = 0
        while i < 60:
            reply = self.__responseMsgCache.get(id)
            if reply:
                self.__responseMsgCache.delete(id)
                return reply["ErrorNo"] == 0
            i += 1
            time.sleep(0.1)
        return False


    def Unsubscribe(self,*subInfos):
        """取消订阅"""
        if not subInfos:
            logger.error('订阅参数不能为空')
            return False
        if not self.__wsClient.IsAvaliable():
            logger.error('行情未连接')
            return False
        req = UnsubscribeReq()
        req.ReqID = uuid.uuid1().hex
        req.RequestType = RequestType.Unsubscribe
        req.Unsubscribes = subInfos
        message = json.dumps(req.__dict__,ensure_ascii=False)
        if not self.__Send(req.ReqID,message):
            logger.error("Unsubscribe market send fail:{0}".format(message))
            return False
        logger.info("Unsubscribe market succeed:{0}".format(message))
        for x in subInfos:
            if x in self.__subscribes:
                self.__subscribes.remove(x)
        return True

    def GetHisBar(self, symbol, exchange, barType, startTime, endTime):
        """获取历史K线数据"""
        if not symbol:
            logger.error("合约代码不能为空")
            return None
        url = Environment.get_instance().get_apiurl('api/MarketData/GetHisBar')
        params = {'symbol':symbol,'exchange':exchange,'dataType':barType,'begin':startTime,'end':endTime}
        resp = httpJsonPost(url,params) 
        strParams = json.dumps(params,ensure_ascii=False)
        if not resp:
            logger.error("response：" + url + "," + strParams)
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("request:" + url + "," + strParams + "response：" + resp)
            return None
        data = js.get('data')
        if not data:
            return None
        result = []
        for x in data:
            bar = self.__ToBar(x)
            result.append(bar)
        return result

    def GetLastBar(self, symbol, exchange, barType, count):
        """获取历史K线数据"""
        if not symbol:
            logger.error("合约代码不能为空")
            return None
        url = Environment.get_instance().get_apiurl('api/MarketData/GetLastBar')
        params = {'symbol':symbol,'exchange':exchange,'dataType':barType,'count':count}
        resp = httpJsonPost(url,params) 
        strParams = json.dumps(params,ensure_ascii=False)
        if not resp:
            logger.error("请求应答为空：" + url + "," + strParams)
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("request:" + url + "," + strParams + "response：" + resp)
            return None
        data = js.get('data')
        if not data:
            return None
        result = []
        for x in data:
            bar = self.__ToBar(x)
            result.append(bar)
        return result

    def GetHisTick(self, symbol, exchange, startTime, endTime):
        """获取历史TICK数据"""
        if not symbol:
            logger.error("合约代码不能为空")
            return None
        url = Environment.get_instance().get_apiurl('api/MarketData/GetHisTick')
        params = {'symbol':symbol,'exchange':exchange,'begin':startTime,'end':endTime}
        resp = httpJsonPost(url,params) 
        strParams = json.dumps(params,ensure_ascii=False)
        if not resp:
            logger.error("请求应答为空：" + url + "," + strParams)
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("request:" + url + "," + strParams + "response：" + resp)
            return None
        data = js.get('data')
        if not data:
            return None
        result = []
        for x in data:
            tick = self.__ToTickData(x)
            result.append(tick)
        return result

    def GetLastTick(self, symbol, exchange, count):
        """获取历史TICK数据"""
        if not symbol:
            logger.error("合约代码不能为空")
            return None
        url = Environment.get_instance().get_apiurl('api/MarketData/GetLastTick')
        params = {'symbol':symbol,'exchange':exchange,'count':count}
        resp = httpJsonPost(url,params) 
        strParams = json.dumps(params,ensure_ascii=False)
        if not resp:
            logger.error("请求应答为空：" + url + "," + strParams)
            return None
        js = json.loads(resp,encoding='utf-8')
        if js.get('error_no') != 0:
            logger.error("request:" + url + "," + strParams + "response：" + resp)
            return None
        data = js.get('data')
        if not data:
            return None
        result = []
        for x in data:
            tick = self.__ToTickData(x)
            result.append(tick)
        return result

    def __ToBar(self,data):
        #消息长度可能会增加
        if not data:
            return None
        bar = BarData()
        bar.symbol = data.get("s")
        bar.exchange = Exchange[data.get("e")]
        bar.bar_type = int(data.get("d"))
        bar.local_time = datetime.datetime.strptime(str(data.get("t")),'%Y%m%d%H%M%S') 
        bar.pre_close = float(data.get("pc"))
        bar.open = float(data.get("o"))
        bar.high = float(data.get("h"))
        bar.low = float(data.get("l"))
        bar.close = float(data.get("c"))
        bar.volume = int(data.get("v"))
        bar.turnover = float(data.get("a"))
        bar.open_interest = int(data.get("oi"))
        bar.settlement = float(data.get("sp"))
        return bar

    def __ToTickData(self,data):
        if not data:
            return None
        tick = TickData()
        tick.symbol = data.get("s")
        tick.exchange = Exchange[data.get("e")]
        tick.local_time = datetime.datetime.strptime(str(data.get("t")),'%Y%m%d%H%M%S') 
        tick.last = float(data.get("px"))
        tick.open = float(data.get("o"))
        tick.high = float(data.get("h"))
        tick.low = float(data.get("l"))
        tick.upper_limit = float(data.get("up"))
        tick.lower_limit = float(data.get("dw"))
        tick.pre_close = float(data.get("pc"))
        tick.volume = int(data.get("v"))
        tick.open_interest = int(data.get("oi"))
        tick.settlement = float(data.get("sp"))
        tick.turnover = float(data.get("a"))
        bids = data.get("bid")
        for x in bids:
            unit = LevelUnit()
            unit.price = float(x['p'])
            unit.volume = int(x['v'])
            tick.bids.append(unit)
        asks = data.get("ask")
        for x in asks:
            unit = LevelUnit()
            unit.price = float(x['p'])
            unit.volume = int(x['v'])
            tick.asks.append(unit)
        return tick

if __name__ == '__main__':

    from iqsopenapi.core.events import *

    config = {
      "runtime": "DEBUG"
    }

    environment = Environment(config)

    event_bus = EventBus()

    event_bus.add_listener(EVENT.On_Bar,lambda msg:logger.info('bar:{0},{1},{2}'.format(msg.data.Symbol,msg.data.LocalTime,msg.data.ClosePx)))

    event_bus.add_listener(EVENT.On_Tick,lambda msg:logger.info('tick:{0},{1},{2}'.format(msg.data.Symbol,msg.data.LocalTime,msg.data.LastPx)))

    api = MarketApi(event_bus)
    api.Init()
    
    init_event = Event(EVENT.On_Init)
    event_bus.publish_event(init_event)

    event_bus.start()

    ret1 = api.Subscribe("rb2010.SHFE.TICK.0","rb2010.SHFE.BAR.300","rb2005.SHFE.TICK.0")

    ret2 = api.Unsubscribe("rb2005.SHFE.TICK.0")

    ret3 = api.GetHisBar("rb2010",Exchange.SHFE,60,"20200423","20200424")

    ret4 = api.GetHisTick("rb2010",Exchange.SHFE,"20200423","20200424")

    ret5 = api.GetLastBar("rb2010",Exchange.SHFE,60,100)

    ret6 = api.GetLastTick("rb2010",Exchange.SHFE,100)

    pass


# -*- coding: utf-8 -*-
import sys
import time
import logging
from logging import handlers
from datetime import datetime
import os

logger = logging.getLogger('iqsopenapilogger')
logger.setLevel(logging.INFO)

logformat = logging.Formatter('%(asctime)-15s %(levelname)s [%(filename)s:%(lineno)d] %(message)s')

console_handler = logging.StreamHandler(stream=sys.stdout)
console_handler.setLevel(logging.DEBUG)
console_handler.setFormatter(logformat)
logger.addHandler(console_handler)

def log2file(filename):
    logger.debug('log filename:{0}'.format(filename))
    path = os.path.dirname(filename)
    if not os.path.exists(path):
        os.makedirs(path)
    trfh = handlers.TimedRotatingFileHandler(filename=filename,when='D',backupCount=30,encoding='utf-8')
    trfh.setLevel(logging.DEBUG)
    trfh.setFormatter(logformat)
    logger.addHandler(trfh)

if __name__ == '__main__':

    log2file(r"D:\logs\test\strategy.log")
    logger.error("test error")
    logger.critical("test critical")
    logger.debug("test debug")
    logger.info("test info")


# -*- coding: utf-8 -*-
from iqsopenapi.core.events import *
import types

class Scheduler(object):

    def __init__(self,event_bus):
        self.__tasks = []
        self.__th = None
        self.__event_bus = event_bus

    def add_task(self,task):
        """添加定时任务"""
        if not task:
            return False
        if self.__th is None:
            self.__th = threading.Thread(target=self.__run)
            self.__th.setDaemon(True)
            self.__th.start()
        self.__tasks.append(task)
        
    def __run(self):
        """定时任务回调"""
        while(True):
            try:
                for task in self.__tasks:
                    task.raise_event_on_time(self.__event_bus)
            except Exception as e:
                logger.exception(e)
            time.sleep(0.2)

class SchedulerTask(object):

    def __init__(self, func,time,event_type,*args):
        self.__func = func
        self.__time = time
        self.__event_type = event_type
        self.__args = args
        self.pre_execute_time = None

    def raise_event_on_time(self,event_bus):
        now = datetime.datetime.now()
        timeStr = now.strftime('%Y%m%d%H%M%S')
        if self.pre_execute_time is not None and self.pre_execute_time.strftime('%Y%m%d%H%M%S') == timeStr:
            return
        if(isinstance(self.__time,datetime.datetime)):
            if self.__time.strftime('%Y%m%d%H%M%S') == timeStr:
                self.__raise_event(now,event_bus)
        elif(isinstance(self.__time,datetime.time)):
            if self.__time.strftime('%H%M%S') == now.strftime('%H%M%S'):
                self.__raise_event(now,event_bus)

    def __raise_event(self,raise_time,event_bus):
        task_event = Event(self.__event_type,func = self.__func,raise_time = raise_time,args = self.__args)
        event_bus.publish_event(task_event)
        self.pre_execute_time = raise_time
        logger.info('run scheduler:' + raise_time.strftime('%Y%m%d%H%M%S'))


# -*- coding: utf-8 -*-
from iqsopenapi.util import *
from iqsopenapi.util.logutil import *
from iqsopenapi.trade.realtrade.message import *
from iqsopenapi.trade.realtrade.Models import *
import uuid
import threading
import time

class PushQuoteSubscriber(object):
    '''实盘推送'''
    def __init__(self, cfg, strategy_id, user):
        '''初始化websocket'''

        self.__cfg = cfg
        self.__strategy_id = strategy_id
        self.__user_info = user
        self.__recv_callback = cfg.recvCallback
        cfg.recvCallback = self.__OnRecv
        cfg.connectedCallback = self.__AuotSubscribe
        self.__wsClient = WebSocketClient(cfg)

    def Connect(self):
        # 连接websocket
        if not self.__wsClient.Connect():
            self.__wirteError("connect failed!")
            return False
        self.__wsClient.AutoHeartBeat()
        return True

    def __SetAccountInfo(self):
        user = self.__user_info
        req = {
            "requestType":RequestType.TradePushConect.name,
            "reqId":uuid.uuid1().hex,
            "brokerAccount":user.AccountID,
            "counterId":user.CounterID,
            "brokerType":user.BrokerType,
        }
        message = json.dumps(req,ensure_ascii=False)
        if not self.__Send(id,message):
            self.__wirteError("set account info fail :{0}".format(message))
            return False
        return True

    def __Send(self,id,msg):
        """消息发送"""
        if not msg: return False
        if not self.__wsClient.Send(msg):
            self.__wirteError("send fail:{0}".format(msg))
            return False
        return True

    def __OnRecv(self,msg):
        '''收到消息'''
        if not msg: return
        # 心跳消息直接返回
        if msg.get("responseType") == ResponseType.HeartBeat.name: return
        call = self.__recv_callback
        if not call: return
        call(msg)

    def __AuotSubscribe(self):
        """自动订阅"""
        try:
            logger.debug("连接成功，开始推送验证信息...")
            if not self.__SetAccountInfo():
                self.__wirteError("subscribe failed!")
            logger.debug("连接成功，推送验证信息完成")
        except Exception as e:
            logger.exception(e)

    def __wirteError(self, error):
        '''写错误日志'''
        logger.error(error)

    def __wirteInfo(self, info):
        '''写日志'''
        logger.info(info)



# -*- coding: utf-8 -*- 

from datetime import  *  
from enum import IntEnum
from iqsopenapi.models.Order import *
from iqsopenapi.models.Contract import *

class Trade(object):
    """成交信息"""

    def __init__(self):
        """成交信息"""

        """订单ID"""
        self.trade_id = ""

        """订单ID"""
        self.order_id = ""

        """策略编号"""
        self.strategy_id = ""

        """账户编号"""
        self.account_id = ""

        """股票代码或合约代码"""
        self.symbol = ""

        """交易所"""
        self.exchange = Exchange.UnKnow

        """委托方向"""
        self.side = OrderSide.Buy

        """委托价格"""
        self.price = 0.0

        """开仓还是平仓 (期货中使用) 非期货为None"""
        self.offset = Offset.UnKnow

        """委托数量"""
        self.quantity = 0

        """成交时间"""
        self.filled_time = datetime.now()


# -*- coding: utf-8 -*-

from iqsopenapi.core.IStrategyLoader import *
from iqsopenapi.core.StrategyLoader import *
from iqsopenapi.core.StrategyContext import *
from iqsopenapi.core.events import *
from iqsopenapi.core.Strategy import *
from iqsopenapi.core.Scheduler import *


# -*- coding: utf-8 -*-
from iqsopenapi.util import *
from websocket import *
from queue import Queue
import datetime
import threading
import time
import json
import ssl

class websocketCfg(object):
    """websocket配置信息"""

    def __init__(self):
        """地址"""
        self.address = ""

        """心跳间隔"""
        self.hbInternal = 5

        """超时"""
        self.keepAliveTimeout = 30

        """心跳内容"""
        self.hbContent = None

        """收到数据回调"""
        self.recvCallback = None

        """"连接成功回调"""
        self.connectedCallback = None

class WebSocketClient(object):
    """WebSocket"""

    def __init__(self,cfg):
        """构造函数"""

        #连接地址
        self.config = cfg
        #websocket
        self.__wsclinet = None
        #上次活跃时间
        self.__lastActiveTime = datetime.datetime.now()
        #缓存队列
        self.__unhandledQueue = Queue()
        t = threading.Thread(target=self.__HandleMsg)
        t.start()
        t1 = threading.Thread(target=self.__ReceiveMsg)
        t1.start()
        self._hasAutoHeartBeat = False
    
    def Connect(self):
        """连接"""
        if self.IsAvaliable():
            return True
        logger.debug("begin connect:{0}".format(self.config.address))
        self.__wsclinet = create_connection(self.config.address,  sslopt={"cert_reqs": ssl.CERT_NONE})
        logger.debug("conenect succeed:{0},state:{1}".format(self.config.address,self.__wsclinet.connected))
        self.__lastActiveTime = datetime.datetime.now()
        if self.config.connectedCallback:
            self.config.connectedCallback()
        return True

    def AutoHeartBeat(self):
        """启动自动心跳功能"""
        if self._hasAutoHeartBeat:
            return
        self._hasAutoHeartBeat = True
        t2 = threading.Thread(target=self.__AutoHeartBeat)
        t2.start()

    def IsAvaliable(self):
        """判断是否是有效连接"""
        if not self.__wsclinet:
            return False
        if not self.__wsclinet.connected:
            return False
        if(datetime.datetime.now() - self.__lastActiveTime).seconds > self.config.keepAliveTimeout:
            return False
        return True

    def Send(self,content):
        """发送消息"""
        if not content:
            return None
        if not self.IsAvaliable():
            return None
        data = bytes(content, encoding = "utf8") 
        return self.__wsclinet.send(data)
 
    def __ReceiveMsg(self):
        """接收消息"""
        while True:
            try:
                 if not self.IsAvaliable():
                     time.sleep(1)
                     continue
                 msg = self.__wsclinet.recv()
                 logger.debug("receive msg from:{0},msg:{1}".format(self.config.address,msg))
                 self.__lastActiveTime = datetime.datetime.now()
                 self.__unhandledQueue.put(msg)
            except Exception as e:
                logger.exception(e)
                time.sleep(1)

    def __HandleMsg(self):
        """处理消息"""
        while True:
            try:
                msg = self.__unhandledQueue.get()
                if not msg:
                    time.sleep(1)
                    continue
                self.__RaiseMessage(msg)
            except Exception as e:
                logger.exception(e)
                time.sleep(1)
    
    def __AutoHeartBeat(self):
        """心跳"""
        if not self.config.hbContent:
            logger.warn("heart beat content is None, skip heart beat:{0}".format(self.config.address))
            return
        logger.debug("start heart beat:{0}...".format(self.config.address))
        while True:
            time.sleep(self.config.hbInternal)
            try:
                if not self.IsAvaliable():
                    logger.error("connect is not avaliable,{0},{1}".format(self.config.address,self.__lastActiveTime))
                    self.__Close()
                    logger.info("begin reconnect:{0}".format(self.config.address))
                    self.Connect()
                    logger.info("reconnect is succeed:{0}".format(self.config.address))
                self.Send(self.config.hbContent)
                logger.debug("send heart beat:{0}".format(self.config.address))
            except Exception as e:
                logger.exception(e)
                self.__Close()

    def __RaiseMessage(self,msg):
        if not msg:
            return
        logger.debug("receive message:{0},{1}".format(msg,self.config.address))
        data = json.loads(msg,encoding='utf-8')
        self.config.recvCallback(data)
    
    def __Close(self):
        try:
            if self.__wsclinet:
                self.__wsclinet.close()
            self.__wsclinet = None
        except Exception as e:
            logger.exception(e)
        
        
if __name__ == '__main__':
    cfg = websocketCfg()
    cfg.address = "wss://dev_apigateway.inquantstudio.com/usermsg/"
    cfg.hbContent = json.dumps({ 'requestType' : 'HeartBeat','reqID' : '' })
    cfg.recvCallback = lambda msg:logger.info(msg)

    ws = WebSocketClient(cfg)
    ws.Connect()



# -*- coding: utf-8 -*-

from iqsopenapi.trade.realtrade.TradeEnums import *
from iqsopenapi.models.Position import *
from iqsopenapi.models.Order import *
from iqsopenapi.models.Account import *

class kv:
    def __init__(self, key, value):
        self.Key = str(key)
        self.Value = str(value)

class Dynamic:
    def __init__(self):
        pass

class ReposeModelBase:
    def __init__(self):
        self.ErrorNo = 0
        self.ErrorInfo = ''
    def ErrorHandle(self, flag):
        if (self.ErrorNo != 0): raise Exception(f'[{flag}错误]-{self.ErrorInfo}')

class FuturesCompanysModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.CompanyInfos = []
        self.ArgeementHerfText = ''
        self.ArgeementHerf = ''
        self.ArgeementTip = ''
        self.AppQRImage = ''

class LoginResult(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.TradeToken = ''
        self.IsSigned = False

class TradeCounterModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.CounterList = []

class SendOrderModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.OrderID = -1

class PositionModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.PositionInfos = []

class EntrustModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.EntrustInfos = []

class ConOrderModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.Data = []

class StopOrderNotTriggerModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.Data = []

class UserAssetsModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.AssetInfo = None

class PositionModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.Positions = []

class DoneModel(ReposeModelBase):
    def __init__(self):
        super().__init__()
        self.Dones = []

class StopOrderNotTrigger:
    def __init__(self):
        self.StopId = -1
        self.Symbol = ''
        self.Name = ''
        self.PosSide = PosSide.Net
        self.PosSideStr = ''
        self.StopType = -1
        self.StopTypeStr = ''
        self.TriggerPrice = 0.0
        self.OrderType = -1
        self.OrderTypeStr = ''
        self.Quantity = -1
        self.StatusStr = '进行中'

class ConditionOrderEntity:
    def __init__(self):
        self.ConditionId = -1
        self.Symbol = ''
        self.ContractId = -1
        self.Status = -1
        self.StatusStr = ''
        self.OrderSide = OrderSide.Buy
        self.OrderSideStr = ''
        self.Offset = Offset.UnKnown
        self.OffSetStr = ''
        self.OrderType = OrderType.LMT
        self.OrderTypeStr = ''
        self.Price = ''
        self.Quantity = -1
        self.ExpireType = ConOrderTimeValidTypeEnum.CurrentTradingDayValid
        self.AddTimeStr = ''
        self.ExpireTimeStr = ''
        self.CondiValues = []
        self.CondiValueStr = ''
        self.OrderNo = -1
        self.OrderRet = ''
        self.OrderNote = ''
        self.TriggerTimeStr = ''

class TradeCounterEntity:
    def __init__(self):
        self.CounterId = -1
        self.CounterName = ''
        self.TradeCounter = TradeCounterType.NONE
        self.ApiList = []

class TradeApiEntity:
    def __init__(self):
        self.ApiAddr = ''
        self.ApiName = ''
        self.ApiInfoId = -1
        self.wsAddr = ''

class CompanyInfo:
    def __init__(self):
        self.CompanyId = -1
        self.IsOnLine = 0
        self.BrokerType = -1
        self.CompanyName = ''
        self.Description = ''
        self.Icon = ''
        self.Phone = ''
        self.OpenAccountAddress = ''
        self.TradeCounter = TradeCounterType.NONE

class UserInfo:
    def __init__(self):
        self.AccountID = ''
        self.BrokerType = -1
        self.PassWord = ''
        self.CounterID = -1
        self.CounterAddr = ''
        self.ApiInfoId = -1
        self.SupInfo = SuperviseInfo()
        self.TradeToken = ''

class SuperviseInfo:
    def __init__(self):
        self.TradeCounter = TradeCounterType.NONE
        self.SysInfo = ''
        self.SysInfoIntegrity = ''
        self.EncrypKeyVersion = ''
        self.ExceptionType = ''


from iqsopenapi import *


def init(context):
    logger.info("init")
    
def handle_tick(context,tick):
    logger.info('from file tick:{0},{1},{2}'.format(tick.Symbol,tick.LocalTime,tick.LastPx))

def handle_bar(context, bar):
    logger.info('bar:{0},{1},{2}'.format(bar.Symbol,bar.LocalTime,bar.LastPx))



# -*- coding: utf-8 -*- 
from iqsopenapi.trade.papertrade.message.IMessage import *

class ReplyResp(ResponseMessage):
    """消息回复"""

    def __init__(self,errorNo,errorInfo):
        """错误代码"""
        self.ErrorNo = errorNo

        """错误消息"""
        self.ErrorInfo = errorInfo

        """应答类型"""
        self.ResponseType = ResponseType.Reply

        """请求编号"""
        self.ReqID = ""






#coding=utf-8
import time
import threading
from iqsopenapi.util.logutil import *
import traceback

class MemCache(object):
    '''内存缓存'''

    def __init__(self):
        '''初始化'''
        self.mem = {}
        self.time = {}
        self.lock = threading.RLock()
        t = threading.Thread(target=self.__checkCache)
        t.start()

    def __checkCache(self):
        """定时检查缓存，清除垃圾数据"""
        while True:
            time.sleep(10)
            if not self.lock.acquire(timeout=0.1):
                logger.error('memcache acquire lock failed!')
                continue
            try:
                for key in list(self.mem.keys()):
                    if self.time[key] != -1 and self.time[key] <= time.time():
                        self.delete(key)
            except Exception as e:
                logger.exception(e)
            finally:
                self.lock.release()

    def set(self, key, data, age=-1):
        '''保存键为key的值，时间位age'''
        if not self.lock.acquire(timeout=0.1):
            logger.error('memcache acquire lock failed!')
            return False
        try:
            self.mem[key] = data
            if age == -1:
                self.time[key] = -1
            else:
                self.time[key] = time.time() + age
            return True
        finally:
            self.lock.release()

    def get(self,key):
        '''获取键key对应的值'''
        if not self.lock.acquire(timeout=0.1):
            logger.error('memcache acquire lock failed!')
            return None
        try:
            if key in self.mem.keys():
                if self.time[key] == -1 or self.time[key] > time.time():
                    return self.mem[key]
                else:
                    self.delete(key)
                    return None
            else:
                return None
        finally:
            self.lock.release()

    def delete(self,key):
        '''删除键为key的条目'''
        if not self.lock.acquire(timeout=0.1):
            logger.error('memcache acquire lock failed!')
            return True
        try:
            if key in self.mem.keys():
                del self.mem[key]
            if key in self.time.keys():
                del self.time[key]
            return True
        finally:
            self.lock.release()

    def clear(self):
        '''清空所有缓存'''
        if not self.lock.acquire(timeout=0.1):
            logger.error('memcache acquire lock failed!')
            return
        try:
            self.mem.clear()
            self.time.clear()
        finally:
            self.lock.release()



# -*- coding: utf-8 -*-
import urllib.request
import ssl
import json

def httpGet(url):
    '''GET请求'''
    ssl._create_default_https_context = ssl._create_unverified_context
    request = urllib.request.Request(url)
    request.add_header('User-Agent', 'Mozilla/5.0 (Windows NT 6.3; WOW64; rv:38.0) Gecko/20100101 Firefox/38.0')
    res_data = urllib.request.urlopen(request,timeout=30)
    res = res_data.read().decode("utf-8")
    return res

def httpJsonPost(reqUrl,dict):
    '''POST请求 application/json'''
    ssl._create_default_https_context = ssl._create_unverified_context   
    data = json.dumps(dict) 
    data = bytes(data,'utf8') 
    request = urllib.request.Request(url = reqUrl,data = data)
    request.add_header('User-Agent', 'Mozilla/5.0 (Windows NT 6.3; WOW64; rv:38.0) Gecko/20100101 Firefox/38.0')
    request.add_header('Content-Type', 'application/json')
    res_data = urllib.request.urlopen(request,timeout=30)
    res = res_data.read().decode("utf-8")
    return res

def httpFormPost(reqUrl,params):
    '''POST请求 application/json'''
    ssl._create_default_https_context = ssl._create_unverified_context   
    data = bytes(params,'utf8') 
    request = urllib.request.Request(url = reqUrl,data = data)
    request.add_header('User-Agent', 'Mozilla/5.0 (Windows NT 6.3; WOW64; rv:38.0) Gecko/20100101 Firefox/38.0')
    request.add_header('Content-Type', 'application/x-www-form-urlencoded')
    res_data = urllib.request.urlopen(request,timeout=30)
    res = res_data.read().decode("utf-8")
    return res

def httpPost(reqUrl,dict):
    '''POST请求'''
    ssl._create_default_https_context = ssl._create_unverified_context   
    data = urllib.parse.urlencode(dict).encode('utf-8')
    request = urllib.request.Request(url = reqUrl,data = data)
    request.add_header('User-Agent', 'Mozilla/5.0 (Windows NT 6.3; WOW64; rv:38.0) Gecko/20100101 Firefox/38.0')
    res_data = urllib.request.urlopen(request,timeout=30)
    res = res_data.read().decode("utf-8")
    return res

if __name__ == '__main__':    
    resp = httpGet('https://stgyapi.inquant.cn/future/Contract/Get?symbol=rb1905&exchange=4')
    print(resp)



# -*- coding: utf-8 -*-
from iqsopenapi.basicdata  import *
from iqsopenapi.core  import *
from iqsopenapi.marketdata  import *
from iqsopenapi.models  import *
from iqsopenapi.trade  import *
from iqsopenapi.util  import *
from iqsopenapi.external import *

def run_file(strategy_file_path, config=None):
    """执行文件"""

    loader = FileStrategyLoader(strategy_file_path)
    return __run(config,loader)

def run_code(source_code, config=None):
    """执行源代码"""

    loader = SourceCodeStrategyLoader(source_code)
    return __run(config,loader)


def run_func(**kwargs):
    """执行自定义函数"""

    config = kwargs.get('config', kwargs.get('__config__', None))
    user_funcs = {
        k: kwargs[k]
        for k in ['on_init', 'on_bar', 'on_tick','on_order','on_trade','on_open','on_close']
        if k in kwargs
    }

    loader = UserFuncStrategyLoader(user_funcs)
    return __run(config,loader)

def __run(config, strategy_loader):
    try:
        environment = Environment(config)
        if environment.config.log:
            logger.setLevel(environment.config.log['level'])
            logutil.log2file(environment.config.log['file'])

        ucontext = StrategyContext()

        event_bus = EventBus()

        environment.scheduler = Scheduler(event_bus)

        environment.market_api = MarketApi(event_bus)

        environment.market_api.Init()

        environment.basicdata_api = BasicDataApi()

        if environment.config.run_type == 'PAPER_TRADING':
            environment.trade_api = PaperTradeApi(event_bus)
            environment.trade_api.Init()
        elif environment.config.run_type == 'LIVE_TRADING':
            environment.trade_api = RealTradeApi(event_bus)
            environment.trade_api.Init()
        elif environment.config.run_type == 'BACKTEST':
            raise Exception('BACKTEST is not support!')

        if environment.config.ext_notify:
            user_notify = ExternalNotify(environment.config.ext_notify['address'],event_bus)
            user_notify.init()
        from copy import copy

        scope = copy(iqsopenapi.models.__dict__)
        scope.update(iqsopenapi.core.StrategyContext.__dict__)

        strategy_loader.load(scope)

        user_strategy = Strategy(event_bus, scope, ucontext)

        user_strategy.init()    

        event_bus.start()

        return True
    except Exception as e:
        logger.exception(e)
        return False



# -*- coding: utf-8 -*- 

from datetime import *
from iqsopenapi.models.Contract import *

class TickData(object):
    """实时数据模型"""

    def __init__(self):

        """股票代码或合约名称"""
        self.symbol = ""

        """交易所"""
        self.exchange = Exchange.UnKnow

        """时间"""
        self.local_time = datetime.now()

        """最新成交价"""
        self.last = 0.0

        """成交量"""
        self.volume = 0

        """持仓量"""
        self.open_interest = 0

        """开盘价"""
        self.open = 0.0

        """最高价"""
        self.high = 0.0

        """最低价"""
        self.low = 0.0

        """总成交额"""
        self.turnover = 0.0

        """结算价"""
        self.settlement = 0.0

        """跌停价"""
        self.lower_limit = 0.0

        """涨停价"""
        self.upper_limit = 0.0

        """昨收"""
        self.pre_close = 0.0

        """买N档"""
        self.bids = []

        """卖N档"""
        self.asks = []

class LevelUnit(object):
    """一档行情"""

    def __init__(self):
        """价格"""
        self.price = 0
        """量"""
        self.volume = 0


# -*- coding: utf-8 -*- 
from enum import IntEnum

class IMessage(object):
    """消息"""

    def __init__(self):
        pass

class IRequestMessage(IMessage):
    """请求消息"""

    def __init__(self):
        """消息类型"""
        self.RequestType = RequestType.HeartBeat

        """请求编号"""
        self.ReqID  = ""

class RequestMessage(IRequestMessage):
    """请求消息"""

    def __init__(self):
        """消息类型"""
        self.RequestType = RequestType.HeartBeat

        """请求编号"""
        self.ReqID = ""

class IResponseMessage(IMessage):
    """应答消息"""

    def __init__(self):
        """消息类型"""
        self.ResponseType = ResponseType.HeartBeat

class ResponseMessage(IResponseMessage):
    """应答消息"""
    
    def __init__(self):
        """消息类型"""
        self.ResponseType = ResponseType.HeartBeat

        """数据"""
        self.Data = None
    

class RequestType(IntEnum):
    """行情消息类型"""
    
    """心跳包消息"""
    HeartBeat = 0

    ''' 设置交易账号等信息 '''
    TradePushConect = 1

class ResponseType(IntEnum):
    """响应消息类型"""

    """心跳包消息"""
    HeartBeat = 0 

    ''' 交易推送 '''
    TradePush = 1

class MsgType(IntEnum):
    ''' 交易推送消息类型 '''

    ''' 委托回报 '''
    Order = 1

    ''' 成交回报 '''
    Trade = 2

    ''' 止盈止损设置 '''
    StopPriceSeting = 3

    ''' 条件单触发 '''
    ConditionOrderTrigger = 4

    ''' 止盈止损触发 '''
    StopOrderTrigger = 5


# -*- coding: utf-8 -*-
from iqsopenapi.environment import Environment
from iqsopenapi.models import *
from iqsopenapi.core.Scheduler import *
from collections import defaultdict
from copy import copy

class StrategyContext(object):
    """
    策略上下文
    """
    def __init__(self):
        """初始化"""
        pass

    def get_strategy_id(self):
        """获取策略编号"""
        if Environment.get_instance().config is None or Environment.get_instance().config.acct_info is None:
            return None
        return Environment.get_instance().config.acct_info['strategy_id']
   
    def subscribe(self,*args):
        """行情订阅"""

        subs = []
        for arg in args:
            if arg[1].upper() == "TICK":
                sub = '{0}.TICK.0'.format(arg[0])
                subs.append(sub)
            else:
                bar_type = self.__get_bar_type(arg[1])
                sub = '{0}.BAR.{1}'.format(arg[0],bar_type)
                subs.append(sub)
        return Environment.get_instance().market_api.Subscribe(*subs)

    def unsubscribe(self,*args):
        """取消订阅"""

        subs = []
        for arg in args:
            if arg[1].upper() == "TICK":
                sub = '{0}.TICK.0'.format(arg[0])
                subs.append(sub)
            else:
                bar_type = self.__get_bar_type(arg[1])
                sub = '{0}.BAR.{1}'.format(arg[0],bar_type)
                subs.append(sub)
        return Environment.get_instance().market_api.Unsubscribe(*subs)

    def get_subscribelist(self):
        """获取订阅列表"""
        return Environment.get_instance().market_api.GetSubscibes()

    def get_contract(self, symbol):
        """获取合约信息"""
        (code,exchange) = self.__split_symbol(symbol)
        data = Environment.get_instance().basicdata_api.GetContract(code, exchange)
        self.__rebuild_data(data)
        return data

    def get_dominant_contract(self, variety):
        """获取主力合约"""
        data = Environment.get_instance().basicdata_api.GetMainContract(variety)
        self.__rebuild_data(data)
        return data

    def get_last_ticks(self, symbol, count):
        """获取最近几笔tick"""
        (code,exchange) = self.__split_symbol(symbol)
        data = Environment.get_instance().market_api.GetLastTick(code, exchange, count)
        if not data:
            return data
        self.__rebuild_data(*data)
        return data

    def get_last_bars(self, symbol, barType, count):
        """获取最近几笔bar"""
        (code,exchange) = self.__split_symbol(symbol)
        bar_type = self.__get_bar_type(barType)
        data = Environment.get_instance().market_api.GetLastBar(code, exchange, bar_type, count)
        if not data:
            return data
        self.__rebuild_data(*data)
        return data

    def get_history_bars(self, symbol, barType, startTime, endTime):
        """获取历史bar"""
        (code,exchange) = self.__split_symbol(symbol)
        bar_type = self.__get_bar_type(barType)
        start = startTime
        end = endTime
        if(isinstance(startTime,datetime.datetime)):
            start = startTime.strftime("%Y%m%d%H%M%S")
            end = endTime.strftime("%Y%m%d%H%M%S")
        elif(isinstance(startTime,datetime.date)):
            start = startTime.strftime("%Y%m%d")
            end = endTime.strftime("%Y%m%d")
        data = Environment.get_instance().market_api.GetHisBar(code, exchange, bar_type, start, end)
        if not data:
            return data
        self.__rebuild_data(*data)
        return data

    def get_history_ticks(self, symbol, startTime, endTime):
        """获取订阅列表"""
        (code,exchange) = self.__split_symbol(symbol)
        start = startTime
        end = endTime
        if(isinstance(startTime,datetime.datetime)):
            start = startTime.strftime("%Y%m%d%H%M%S")
            end = endTime.strftime("%Y%m%d%H%M%S")
        elif(isinstance(startTime,datetime.date)):
            start = startTime.strftime("%Y%m%d")
            end = endTime.strftime("%Y%m%d")
        data = Environment.get_instance().market_api.GetHisTick(code, exchange, start, end)
        if not data:
            return data
        self.__rebuild_data(*data)
        return data

    def insert_order(self, symbol, order_side, order_type, offset, price, quantity):
        """下单"""
        (code,exchange) = self.__split_symbol(symbol)
        data = Environment.get_instance().trade_api.SendOrder(code,exchange,order_side,price,quantity,order_type,offset)
        self.__rebuild_data(data)
        return data

    def insert_smart_order(self, symbol, quantity, price=None, today_first=True):
        """智能报单
        symbol—合约代码, str
        quantity—委托手数，正值表示买入，负值表示卖出, int
        当前多仓，正数：开多仓
        当前多仓，负数：平多仓，超过头寸数量反方向开空仓
        当前空仓，正数：平空仓，超过头寸数量反方向开多仓
        当前空仓，负数：开空仓

        当前存在双向单，正数：平空仓，超过空头头寸数量开多仓
        当前存在双向单，负数：平多仓，超过多头头寸数量开空仓

        price—委托价格，None表示市价单，否则为限价单, float
        today_first—平仓优先级，默认优先平今仓，超出部分平老仓
        """
        if quantity == 0:
            raise Exception('smart order quantity must not be less than 0!')
        order_type = OrderType.LMT
        if price is None:
            price = 0.0
            order_type = OrderType.MKT  
        pos = self.__get_smart_order_pos(symbol,quantity)
        if pos is None:
            if quantity > 0:
                return self.insert_order(symbol,OrderSide.Buy,order_type,Offset.Open,price,quantity)
            else:
                return self.insert_order(symbol,OrderSide.Sell,order_type,Offset.Open,price,-quantity)
        if pos.side == PosSide.Net:
            raise Exception('smart order only support long or short position!')
        if pos.side == PosSide.Long and quantity > 0:
            return self.insert_order(symbol,OrderSide.Buy,order_type,Offset.Open,price,quantity)
        elif pos.side == PosSide.Long and quantity < 0:
            closeQty = min(pos.quantity,-quantity)
            order = None
            if today_first:
                if pos.today_qty > 0:
                    order = self.insert_order(symbol,OrderSide.Sell,order_type,Offset.CloseToday,price,min(closeQty,pos.today_qty))
                closeYesQty = closeQty - min(closeQty,pos.today_qty)
                if closeYesQty > 0:
                    order = self.insert_order(symbol,OrderSide.Sell,order_type,Offset.CloseYesterday,price,closeYesQty)
            else:
                yesQty = pos.quantity - pos.today_qty
                if yesQty > 0:
                    order = self.insert_order(symbol,OrderSide.Sell,order_type,Offset.CloseYesterday,price,min(closeQty,yesQty))
                closeTodayQty = closeQty - min(closeQty,yesQty)
                if closeTodayQty > 0:
                    order = self.insert_order(symbol,OrderSide.Sell,order_type,Offset.CloseToday,price,closeTodayQty)
            if -quantity - closeQty > 0:
                return self.insert_order(symbol,OrderSide.Sell,order_type,Offset.Open,price,-quantity - closeQty)
            else:
                return order
        elif pos.side == PosSide.Short and quantity > 0:
            closeQty = min(pos.quantity,quantity)
            order = None
            if today_first:
                if pos.today_qty > 0:
                    order = self.insert_order(symbol,OrderSide.Buy,order_type,Offset.CloseToday,price,min(closeQty,pos.today_qty))
                closeYesQty = closeQty - min(closeQty,pos.today_qty)
                if closeYesQty > 0:
                    order = self.insert_order(symbol,OrderSide.Buy,order_type,Offset.CloseYesterday,price,closeYesQty)
            else:
                yesQty = pos.quantity - pos.today_qty
                if yesQty > 0:
                    order = self.insert_order(symbol,OrderSide.Buy,order_type,Offset.CloseYesterday,price,min(closeQty,yesQty))
                closeTodayQty = closeQty - min(closeQty,yesQty)
                if closeTodayQty > 0:
                    order = self.insert_order(symbol,OrderSide.Buy,order_type,Offset.CloseToday,price,closeTodayQty)
            if quantity - closeQty > 0:
                return self.insert_order(symbol,OrderSide.Buy,order_type,Offset.Open,price,quantity - closeQty)
            else:
                return order
        elif pos.side == PosSide.Short and quantity < 0:
            return self.insert_order(symbol,OrderSide.Sell,order_type,Offset.Open,price,-quantity)

        raise Exception('smart order error!')

    def __get_smart_order_pos(self,symbol, quantity):
        '''获取智能单持仓'''
        all_pos = self.get_positions()
        if symbol not in all_pos:
             return None
        if len(all_pos[symbol]) >= 2:
            if quantity > 0:
                if PosSide.Short in all_pos[symbol]:
                    return all_pos[symbol][PosSide.Short]
                raise Exception('smart order not exist short position!')
            elif quantity < 0:
                if PosSide.Long in all_pos[symbol]:
                    return all_pos[symbol][PosSide.Long]
                raise Exception('smart order not exist long position!')
            else:
                raise Exception('smart order quantity must not be less than 0!')
        else:
            return list(all_pos[symbol].values())[0]

    def cancel_order(self, order):
        """撤单"""
        data = Environment.get_instance().trade_api.CancelOrder(order)
        self.__rebuild_data(data)
        return data

    def get_orders(self):
        """获取当日委托"""
        data = Environment.get_instance().trade_api.GetOrders()
        if not data:
            return data
        self.__rebuild_data(*data)
        return data

    def get_trades(self):
        """获取当日成交"""
        data = Environment.get_instance().trade_api.GetTrades()
        if not data:
            return data
        self.__rebuild_data(*data)
        return data

    def get_order(self, order_id):
        """获取当日委托"""
        data = Environment.get_instance().trade_api.GetOrder(order_id)
        self.__rebuild_data(data)
        return data

    def get_open_orders(self):
        """获取Open委托"""
        data = Environment.get_instance().trade_api.GetOpenOrders()
        if not data:
            return data
        self.__rebuild_data(*data)
        return data

    def get_positions(self):
        """获取持仓"""
        data = Environment.get_instance().trade_api.GetPositions()
        if not data:
            return defaultdict(dict)
        self.__rebuild_data(*data)
        val = defaultdict(dict)
        for x in data:
            val[x.symbol][x.side] = x
        return dict(val)

    def get_position(self, symbol):
        """获取持仓"""
        all_pos = self.get_positions()
        if symbol not in all_pos:
             return {PosSide.Long:None,PosSide.Short:None}
        val = copy(all_pos[symbol])
        if PosSide.Long not in val:
            val[PosSide.Long] = None
            return val
        if PosSide.Short not in val:
            val[PosSide.Short] = None
            return val
        return val

    def get_account(self):
        """获取订阅列表"""
        return Environment.get_instance().trade_api.GetAccount()

    def run_by_time(self,func,time,*args):
        """定时运行函数"""
        return Environment.get_instance().scheduler.add_task(SchedulerTask(func,time,EVENT.On_Scheduler,args))

    def insert_func_at_time(self,func,time,*args):
        """指定时间运行函数"""
        return Environment.get_instance().scheduler.add_task(SchedulerTask(func,time,EVENT.On_Scheduler,args))

    def get_datetime(self):
        """获取当前时间"""
        return datetime.datetime.now()

    def __split_symbol(self,symbol):
        if symbol is None or symbol == '':
            raise Exception('illegal symbol:' + symbol)
        arr = symbol.split('.')
        if len(arr) != 2:
            raise Exception('illegal symbol:' + symbol)
        return (arr[0],Exchange[arr[1]])

    def __get_bar_type(self,bartype):
        if bartype is None or bartype == '':
            raise Exception('illegal bartype:' + bartype)
        if bartype.isdigit():
            return bartype
        multi = int(bartype[:-1])
        unit = 0
        if bartype[-1] == 's':
            unit = 1
        elif bartype[-1] == 'm':
            unit = 60
        elif bartype[-1] == 'h':
            unit = 3600
        elif bartype[-1] == 'd':
            unit = 86400
        else:
            raise Exception('illegal bartype:' + bartype)
        return multi * unit

    def __rebuild_data(self, *data):
        if data is None:
            return
        for x in data:
            if hasattr(x,'symbol') and '.' not in x.symbol:
                x.symbol = '{0}.{1}'.format(x.symbol, x.exchange.name)


# -*- coding: utf-8 -*- 

from datetime import *
from enum import IntEnum

class Contract(object):
    """合约信息"""

    def __init__(self):
        """代码"""
        self.symbol = ""

        """名称"""
        self.contract_name = ""

        """交易所"""
        self.exchange = Exchange.UnKnow

        """投资品种类型"""
        self.contract_type = ContractType.UnKnow

        """每股手数"""
        self.lots = 1

        """最小价差"""
        self.step = 0

        """到期日"""
        self.expire_date = datetime.now()

        """行权价"""
        self.strike_px = 0.0

        """期权类型 P:Put C:Call"""
        self.right = ""

        """上市日期"""
        self.list_date = datetime.now()

        """币种"""
        self.currency = Currency.CNY

        """交易时间"""
        self.trade_times = []

class tradeTime(object):
    """交易时间"""

    def __init__(self):
        """开始时间"""
        self.begin = ""

        """结束时间"""
        self.end = ""

class ContractType(IntEnum):
        """合约类型"""

        """未知"""
        UnKnow = 0
        """股票"""
        Stock = 1
        """期货"""
        Future = 2
        """期权"""
        Option = 3

class Exchange(IntEnum):
        """交易所"""

        """未知"""
        UnKnow = 0
        """上交所"""
        SHSE = 1
        """深交所"""
        SZSE = 2
        """中金所"""
        CFFEX = 3
        """上期所"""
        SHFE = 4
        """大商所"""
        DCE = 5
        """郑商所"""
        CZCE = 6
        """港交所"""
        HKSE = 7
        """纳斯达克"""
        NASDAQ = 8
        """纽约证券交易所"""
        NYSE = 9
        """全美证券交易所"""
        AMEX = 10
        """新三板"""
        SBSE = 11
        """伦敦商品交易所"""
        LME = 12
        """马来西亚衍生产品交易所"""
        BMD = 13
        """东京商品交易所"""
        TOCOM = 14
        """上海国际能源交易中心"""
        INE = 15

class Currency(IntEnum):
        """币种"""
        
        """人民币"""
        CNY = 0
        """美元"""
        USD = 1
        """港币"""
        HKD = 2


# -*- coding: utf-8 -*-
from iqsopenapi.util.HttpUtil import *
from iqsopenapi.util.logutil import *
from iqsopenapi.util.MemCache import *
from iqsopenapi.util.DES3Cryptogram import *
from iqsopenapi.util.WebSocketClient import *
from iqsopenapi.util.Util import *



# -*- coding: utf-8 -*-
from iqsopenapi.core  import *
from iqsopenapi.util.HttpUtil import *
from iqsopenapi.util.logutil import *
from iqsopenapi.util.DES3Cryptogram import *
from iqsopenapi.trade.ITradeApi import *
from iqsopenapi.models.Order import *
from iqsopenapi.models.Account import *
from iqsopenapi.models.Position import *
from urllib.parse import quote
import string
import json
import time
import uuid
import datetime
from concurrent.futures.thread import ThreadPoolExecutor
from iqsopenapi.trade.realtrade.RealTradeApiCore import *
from iqsopenapi.trade.realtrade.Models import *
from iqsopenapi.trade.realtrade.PushQuoteSubscriber import *
from iqsopenapi.trade.realtrade.message import *
from copy import copy

class RealTradeApi(ITradeApi):
    """期货交易"""

    def __init__(self,event_bus):
        """构造函数"""
        super(RealTradeApi,self).__init__(event_bus)
        
        self.__event_bus = event_bus

        self.__api_core = RealTradeApiCore()
        self.__user_info = None

    def Init(self):
        """初始化"""
        user = UserInfo()

        info = Environment.get_instance().config.acct_info

        # 设置策略id
        strategy_id = info['strategy_id']
        self.__strategy_id = strategy_id

        # 初始化user变量
        brokerType = info['broker_type']
        counterId = info['comp_counter']
        crypt =('crypt' in info) and info['crypt']
        aid = info['account']
        psd = info['password']
        if crypt:
            user.AccountID = decrypt(aid)
            user.PassWord = decrypt(psd)
        else:
            user.AccountID = aid
            user.PassWord = psd
        user.BrokerType = brokerType
        user.CounterID = counterId

        # 按照指定的期货公司id和柜台id获取对应的柜台
        api = self.__api_core
        counterInfo = api.GetCounterInfo(brokerType)
        counters = list(filter(lambda x:x.CounterId == counterId, counterInfo.CounterList))
        if(None ==  counters or len(counters) == 0):raise Exception("没有对应的柜台id")
        counter = counters[0]
        counterApi = api.GetCounterApi(counter)

        # 设置user变量的柜台信息
        user.SupInfo.TradeCounter = counter.TradeCounter
        user.ApiInfoId = counterApi.ApiInfoId
        user.CounterAddr = counterApi.ApiAddr

        # 登录并设置user变量的TradeToken
        self.__wirteInfo('登录!')
        login = api.AccountLogin(user)
        if (None ==  login) : raise Exception('登录失败!')
        user.TradeToken = login.TradeToken

        # 保存user变量
        self.__user_info = user
        api.SetUserInfo(user)

        self.__startReLoginMonitor()
        
        # 设置推送
        self.__wirteInfo('设置推送!')
        cfg = websocketCfg()
        cfg.address = counterApi.wsAddr
        cfg.hbContent = json.dumps({ 'requestType' : RequestType.HeartBeat.name,'reqId' : '' })
        cfg.recvCallback = self.__OnRecv
        subscriber = PushQuoteSubscriber(cfg, strategy_id, user)
        subscriber.Connect()
        self.__subscriber = subscriber

        self.__wirteInfo('初始化完成!')
        return True

    def SendOrder(self, symbol, exchange, orderSide, price, quantity, orderType, offset):
        """下单"""
        if not symbol or symbol == '':
            self.__wirteError("symbol不能为空")
            return ResultInfo(-1,"symbol不能为空")
        if not isinstance(quantity, int):
            self.__wirteError("quantity必须为整数:{0},{1}".format(quantity,symbol))
            return ResultInfo(-1,"quantity必须为整数:{0},{1}".format(quantity,symbol))
        self.__wirteInfo("准备下单，symbol:{0},exchange:{1},orderSide:{2},offset:{3},orderType:{4},price:{5},quantity:{6}".format(symbol, exchange.name, orderSide, offset.name, orderType.name, price, quantity))
        
        order = Order()
        order.exchange = exchange
        order.filled = 0
        order.note = ''
        order.offset = offset
        order.order_id = ''
        order.side = orderSide
        order.order_time = datetime.now()
        order.order_type = orderType
        order.price = price
        order.quantity = quantity
        order.status = OrderStatus.NotSent
        order.symbol = symbol
        order.strategy_id = self.__strategy_id
        order.account_id = self.__user_info.AccountID
        order_event = Event(EVENT.On_Order,data = order)
        self.__event_bus.publish_event(order_event)
        
        params = {
            'symbol':symbol,
            'exchange':int(exchange),
            'orderSide':int(orderSide),
            'price':price,
            'quantity':quantity,
            'orderType':int(orderType),
            'offset':int(offset),
            'strategyId':self.__strategy_id,
            'tradeAccount':self.__user_info.AccountID
        }
        self.__wirteInfo("begin sendorder，param:{0}".format(json.dumps(params,ensure_ascii=False)))

        model = self.__api_core.SendOrder(order)
        result = model.ErrorNo == 0
        self.__wirteInfo(f'sendorder complated，param:{json.dumps(params,ensure_ascii=False)},resp:{result}')

        neworder = copy(order)
        if(not result):
            neworder.note = model.ErrorInfo
            neworder.status = OrderStatus.Rejected
        else:
            neworder.status = OrderStatus.Sended
            neworder.order_id = str(model.OrderID)
        neworder_event = Event(EVENT.On_Order,data = neworder)
        self.__event_bus.publish_event(neworder_event)
        return neworder

    def CancelOrder(self,order):
        """撤单"""
        if not order or not order.order_id:
            self.__wirteInfo("invalid order can't cancel")
            return None

        cancelOrder = copy(order)
        cancelOrder.status = OrderStatus.PendingCancel
        cancelOrder.strategy_id = self.__strategy_id
        cancelOrder.account_id = self.__user_info.AccountID
        order_event = Event(EVENT.On_Order,data = cancelOrder)
        self.__event_bus.publish_event(order_event)

        params = {'orderId':cancelOrder.order_id,'strategyId':self.__strategy_id,'tradeAccount': self.__user_info.AccountID}
        logger.info("begin cancelorder，param:{0}".format(json.dumps(params,ensure_ascii=False)))
        
        model = self.__api_core.CancelOrder(cancelOrder.order_id)
        result = model.ErrorNo == 0

        newCancelOrder = self.GetOrder(order.order_id)

        newCancelOrder_event = Event(EVENT.On_Order,data = newCancelOrder)
        self.__event_bus.publish_event(newCancelOrder_event)

        return newCancelOrder

    def GetAccount(self):
        """获取资产信息"""
        model = self.__api_core.GetAccountFund()
        if(None == model):raise Exception("获取资产信息失败")
        asset = model.AssetInfo
        asset.strategy_id = self.__strategy_id
        return asset

    def GetOrder(self,orderId):
        """根据内联ID号 获取订单详情"""
        orders = self.GetOrders()
        if not orders: return None
        for x in orders:
            if x.order_id == orderId:
                return x
        return None

    def GetOpenOrders(self):
        """获取打开的订单"""
        orders = self.GetOrders()
        if not orders: return None
        openOrders = []
        for x in orders:
            if x.isopen():
                openOrders.append(x)
        return openOrders

    def GetOrders(self):
        """获取当日委托"""
        orders = self.__api_core.GetEntrustInfos()
        if(None ==  orders):raise Exception("获取当日委托失败")
        for order in orders:
            order.account_id = self.__user_info.AccountID
            order.strategy_id = self.__strategy_id
        return orders

    def GetPositions(self):
        """获取持仓"""
        positions = self.__api_core.GetPositionInfos()
        if(None == positions):raise Exception("获取持仓失败")
        for position in positions:
            position.account_id = self.__user_info.AccountID
            position.strategy_id = self.__strategy_id
        return positions

    def GetTrades(self):
        """获取当日成交"""
        dones = self.__api_core.GetTrades()
        if(None == dones):raise Exception("获取当日成交失败")
        for done in dones:
            done.account_id = self.__user_info.AccountID
            done.strategy_id = self.__strategy_id
        return dones

    def __OnRecv(self,msg):
        '''收到消息'''
        if not msg:
            return
        if msg.get("responseType") != ResponseType.TradePush.name:
            return

        logger.info(msg)
        msg_type = MsgType(msg.get('msgType'))
        data = msg.get('metaData')

        if msg_type == MsgType.Order:
            order = self.__ToOrder(data)
            if not order:
                return
            order_event = Event(EVENT.On_Order,data = order)
            self.__event_bus.publish_event(order_event)
        if msg_type == MsgType.Trade:
            trade = self.__ToTrade(data)
            if not trade:
                return
            trade_event = Event(EVENT.On_Trade,data = trade)
            self.__event_bus.publish_event(trade_event)

    def __ToOrder(self, item):
        data = json.loads(item)
        order = Order()
        order.exchange = Exchange(data['exchange'])
        order.order_time = datetime.strptime(data['orderTime'],'%Y-%m-%d %H:%M:%S')
        #order.filled = 
        order.note = str(data['errorMsg'])
        order.offset = Offset(data['offset'])
        order.side = OrderSide.__dict__[data['orderSide']]
        order.order_type = OrderType(data['orderType'])
        order.price = float(data['price'])
        order.quantity = int(data['quantity'])
        order.status = OrderStatus(data['status'])
        order.symbol = str(data['symbol'])
        order.strategy_id = self.__strategy_id
        order.account_id = self.__user_info.AccountID
        order.order_id = data['orderId']
        if ('errorNo' in data):
            errorNo = int(data['errorNo'])
            if(errorNo != 0 and 'errorMsg' in data):
                order.note = data['errorMsg']
        return order

    def __ToTrade(self, item):
        data = json.loads(item)
        trade = Trade()
        trade.strategy_id = self.__strategy_id
        trade.account_id = self.__user_info.AccountID
        trade.exchange = Exchange(data['exchange'])
        trade.symbol = str(data['symbol'])
        trade.order_id = str(data['orderId'])
        trade.trade_id = str(data['tradeId'])
        trade.side = OrderSide.__dict__[data['orderSide']]
        trade.price = float(data['price'])
        trade.offset = Offset(data['offset'])
        trade.quantity = int(data['filled'])
        trade.filled_time = datetime.strptime(data['filledTime'],'%Y-%m-%d %H:%M:%S')
        return trade

    def __startReLoginMonitor(self):
        model = self.__api_core.GetAccountFund()
        if(self.__needReLogin(model)):
            self.__reLogin()
        Timer(30, self.__startReLoginMonitor).start()
    
    def __reLogin(self):
        self.__wirteInfo('重新登录!')
        api = self.__api_core
        user = self.__user_info
        login = api.AccountLogin(user)
        if (None ==  login) : return
        user.TradeToken = login.TradeToken
        self.__user_info = user
        api.SetUserInfo(user)

    def __needReLogin(self, model):
        return model != None and model.ErrorNo == 1

    def __wirteError(self, error):
        '''写错误日志'''
        logger.error(error)

    def __wirteInfo(self, info):
        '''写日志'''
        logger.info(info)

def __object_2_json(x):
    return x.__dict__

if __name__ == '__main__':
    """测试"""
    try:
        raise Exception('未测试')
        #logutil.setLogPath("d:/logs/")
        event_bus = EventBus()
        trade_api = RealTradeApi(event_bus)
        trade_api.Init()

        position = trade_api.GetPositions()
        orders = trade_api.GetOrders()
        assets = trade_api.GetAccount()
        trades = trade_api.GetTrades()

        opens = trade_api.GetOpenOrders()
        if(opens and len(opens) > 0):
            for open in opens:
                cancel = trade_api.CancelOrder(open.order_id)
                if (None == cancel) : raise Exception("撤单失败")

        symbol = 'c2007'
        exchange = Exchange.DCE
        orderSide = OrderSide.Buy
        offset = Offset.Open
        orderType = OrderType.LMT
        price = 1955
        quantity = 1

        sendOrder = trade_api.SendOrder(symbol, exchange, orderSide, price, quantity, orderType, offset)
        if (None == sendOrder) : raise Exception("下单失败")
        aa = price

    except Exception as e:
        print(e)
    pass



# -*- coding: utf-8 -*-
import ctypes
import os
import sys
from ctypes import *
from iqsopenapi.trade.realtrade.TradeEnums import TradeCounterType
from iqsopenapi.trade.realtrade.Models import kv

class SupervisionHelper:
    def __init__(self):

        if self.__isLinux():
            self.__libpath = os.path.join(os.path.dirname(__file__), 'linux')
        else:
            if self.__isX64():
                self.__libpath = os.path.join(os.path.dirname(__file__), 'win','x64')
            else:
                self.__libpath = os.path.join(os.path.dirname(__file__), 'win','x86')
        os.chdir(self.__libpath)

    def string_buffer2string(self, buffer, length, code='ascii'):
        return buffer.value[:length].decode(code).strip()
        
    def create_string_buffer(self, length):
        return ctypes.create_string_buffer(length)

    def dingdian(self):
        dingdian = cdll.LoadLibrary('InformationCollect.dll')
        length = 1024
        infoSb = self.create_string_buffer(length)
        cryptVerSb = self.create_string_buffer(length)
        result= dingdian.apexsoft_getsysteminfo(infoSb, length, cryptVerSb, length)
        info = self.string_buffer2string(infoSb, length)
        crypt = self.string_buffer2string(cryptVerSb, length)
        return result, info, crypt
        
    def ctp(self):
        length = 1024
        array = (c_byte * length)()
        len = (c_int*1)()
        if self.__isLinux():
            func_name = '_Z21CTP_GetRealSystemInfoPcRi'
            absolute_dllfile_path = os.path.join(self.__libpath, 'LinuxDataCollect.so')
            ctp = CDLL(absolute_dllfile_path)
        else:
            ctp = CDLL('WinDataCollect.dll')
            if self.__isX64():
                func_name = '?CTP_GetSystemInfo@@YAHPEADAEAH@Z'
            else:
                func_name = '?CTP_GetSystemInfo@@YAHPADAAH@Z'
        result = ctp[func_name](array, len)
        temp_list = array[:len[0]]
        info = ','.join(list(map(str, temp_list)))
        return result, info
        
    def hundsun(self):
        #stdcall约定用windll加载
        hs = windll.LoadLibrary('HsFutuSystemInfo.dll')
        length = 1024
        infoSb = self.create_string_buffer(length)
        integritySb = self.create_string_buffer(length)
        infoLength = (c_int*1)(length)
        integrityLength = (c_int*1)(length)
        result = hs.hundsun_getsysteminfo(infoSb, infoLength, integritySb, integrityLength)
        info = self.string_buffer2string(infoSb, infoLength[0])
        integrity = self.string_buffer2string(integritySb, integrityLength[0])
        return result, info, integrity
        
    def jsd(self):
        jsd = cdll.LoadLibrary('KCC_API.dll')
        length = 1024
        infoSb = self.create_string_buffer(length)
        infoLength = (c_int*1)(length)
        result = jsd.KingStar_GetSystemInfo(infoSb, infoLength)
        if(result != 0): raise Exception('获取JSD穿透式监管信息时出错')
        info = self.string_buffer2string(infoSb, infoLength[0])
        return result, info
        
    def fm(self):
        fm = cdll.LoadLibrary('FMCollectWrapper.dll')
        length = 1024
        infoSb = self.create_string_buffer(length)
        infoLength = (c_int*1)(length)
        flagPoint = (c_int*1)()
        result = fm.StaticGetUserLocalSystemInfo(infoSb, length, infoLength, flagPoint)
        if(result != 0): raise Exception('获取FM穿透式监管信息时出错')
        info = self.string_buffer2string(infoSb, infoLength[0])
        return result, info, flagPoint[0]

    def SetInfo(self, supInfo):
        info = ''
        integrity = ''
        ver = ''
        except_code = 0
        json = []
        if (supInfo.TradeCounter == TradeCounterType.CTP):
            except_code, info = self.ctp()
            json.append(kv('sysInfo', info))
            json.append(kv('exceptionType', except_code))
        elif (supInfo.TradeCounter == TradeCounterType.HS):
            except_code, info, integrity = self.hundsun()
            json.append(kv('sysInfo', info))
            json.append(kv('sysInfoIntegrity', integrity))
            json.append(kv('exceptionType', except_code))
        elif (supInfo.TradeCounter == TradeCounterType.DD):
            except_code, info, ver = self.dingdian()
            json.append(kv('sysInfo', info))
            json.append(kv('encrypKeyVersion', ver))
            json.append(kv('exceptionType', except_code))
        elif (supInfo.TradeCounter == TradeCounterType.JSD):
            except_code, info= self.jsd()
            json.append(kv('sysInfo', info))
            json.append(kv('exceptionType', except_code))
        elif (supInfo.TradeCounter == TradeCounterType.FM):
            except_code, info, flag= self.fm()
            json.append(kv('sysInfo', info))
            json.append(kv('exceptionType', except_code))
        else : raise Exception('不支持的穿透式监管类型!')
        supInfo.SysInfo = info
        supInfo.EncrypKeyVersion = ver
        supInfo.SysInfoIntegrity = integrity
        supInfo.ExceptionType = str(except_code)
        return json

    def __isX64(self):
        return sys.maxsize > 2**32

    def __isLinux(self):
        return sys.platform == "linux" or sys.platform == "linux2"

if __name__ == '__main__':
    #Supervision文件夹放到此文件目录下
    helper = SupervisionHelper()
    
    #dingdian
    print('dingdian')
    result,info,crypt = helper.dingdian()
    print(f'info:{info}\ncrypt:{crypt}\nresult:{result}')

    #ctp
    print('\n\nctp')
    result,info = helper.ctp()
    print(f'info:{info}\nresult:{result}')

    #hundsun
    print('\n\nhundsun')
    result,info,integrity = helper.hundsun()
    print(f'info:{info}\nintegrity:{integrity}\nresult:{result}')

    #jsd
    print('\n\njsd')
    result,info = helper.jsd()
    print(f'info:{info}\nresult:{result}')

    #fm
    print('\n\nfm')
    result,info,flag = helper.fm()
    print(f'info:{info}\nflag:{flag}\nresult:{result}')


# -*- coding: utf-8 -*-
import time
import sys
import os
import json
import datetime

def ExecutedTime(func):
    start = time.time()
    result = func()
    end = time.time()
    escape = end - start
    return escape, result

def Minimum(values, func):
    value = None
    last = None
    for x in values:
        result = func(x)
        if not last or last > result:
            value = x
            last = result
    return value

class DateTimeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime('%Y-%m-%d')
        return json.JSONEncoder.default(self, obj)